/**
 * nSX2 - an OpenSource PS2 Emulator
 * Copyright (C) 2002 Muad
 *
 * homepage: http://nsx2.emulation64.com
 * email address: atreides.muaddib@caramail.com
 * 
 * If you want to contribute to the project please contact
 * me first (maybe someone is already making what you are
 * planning to do).
 *
 *
 * This program is free software; you can redistribute it and/
 * or modify it under the terms of the GNU General Public Li-
 * cence as published by the Free Software Foundation; either
 * version 2 of the Licence, or any later version.
 *
 * This program is distributed in the hope that it will be use-
 * ful, but WITHOUT ANY WARRANTY; without even the implied war-
 * ranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public Licence for more details.
 *
 * You should have received a copy of the GNU General Public
 * Licence along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
**/
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>

#include "nSX2.h"

#include "main.h"
#include "pluggins_main.h"
#include "PS2Edefs.h"
#include "resource.h"
#include "config.h"


#include "GUI_LogFunctions.h"



#define MAX_NUMBER_PLUGINS 20
#define MAX_LENGTH_PLUGINS 40




char GFXPLUGINS [MAX_NUMBER_PLUGINS][MAX_LENGTH_PLUGINS] ;
char INPUTPLUGINS [MAX_NUMBER_PLUGINS][MAX_LENGTH_PLUGINS] ;
char SOUNDPLUGINS [MAX_NUMBER_PLUGINS][MAX_LENGTH_PLUGINS] ;
char SIFPLUGINS [MAX_NUMBER_PLUGINS][MAX_LENGTH_PLUGINS] ;
char DBGPLUGINS [MAX_NUMBER_PLUGINS][MAX_LENGTH_PLUGINS] ;

int FindPluginToSelect(char * path,long type)
{// Search plugin inside plugin arrays,return index,if not found return 0.
    int temp ;
    
    if (strcmp(path,"")==0) return 0;
    
    switch(type)
    {
        case PS2E_LT_GS:
           for (temp=0;temp<MAX_NUMBER_PLUGINS;temp++)
           {
                if (strcmp(path,GFXPLUGINS[temp])==0)
                {
                     return temp;           
                }
           }
           break;     
                
        case PS2E_LT_PAD:
           for (temp=0;temp<MAX_NUMBER_PLUGINS;temp++)
           {
                if (strcmp(path,INPUTPLUGINS[temp])==0)
                {
                     return temp;           
                }
           }
           break;     
        
        
        case PS2E_LT_SPU2:
           for (temp=0;temp<MAX_NUMBER_PLUGINS;temp++)
           {
                if (strcmp(path,SOUNDPLUGINS[temp])==0)
                {
                     return temp;           
                }
           }
           break;     

        case PS2E_LT_DBG:
           for (temp=0;temp<MAX_NUMBER_PLUGINS;temp++)
           {
                if (strcmp(path,DBGPLUGINS[temp])==0)
                {
                     return temp;           
                }
           }
           break;     



     }
    return 0;
}

void getAppFullPath (char *ret) {
    char drive[_MAX_DRIVE], dirn[_MAX_DIR] ;
	char fname[_MAX_FNAME], ext[_MAX_EXT] ;
	char path_buffer[_MAX_DIR] ;
	
	GetModuleFileName(NULL, path_buffer, sizeof(path_buffer)); 
    _splitpath(path_buffer, drive, dirn, fname, ext);
    strcpy(ret, drive);
    strcat(ret, dirn);
}

BOOL CALLBACK ConfigureDlgProc(HWND hwnd, UINT Msg, WPARAM wParam, LPARAM lParam) 
{ 
    int ind = 0;
    _PS2EgetLibType    PS2E_GetLibType;
	_PS2EgetLibName    PS2E_GetLibName;
	_PS2EgetLibVersion PS2E_GetLibVersion;
	char tmpStr[256],tmpStr2[256] ;
	WIN32_FIND_DATA FindData ;
    HANDLE Find;
	HANDLE Lib;
	PLUGIN_INFO sif_plugin_info;
   
    switch (Msg) 
    { 
		case WM_INITDIALOG:
            {
				
				sprintf(tmpStr,"Configure Plugins");
				SetWindowText(hwnd,tmpStr) ;
				getAppFullPath(tmpStr) ; 
                strcat(tmpStr, "plugins/*.dll");
				Find = FindFirstFile(tmpStr, &FindData);
				do{
				    sprintf(tmpStr2,"%s%s", "plugins/", FindData.cFileName);
				    
				    Lib = LoadLibrary(tmpStr2);
				    PS2E_GetLibType = (_PS2EgetLibType) GetProcAddress((HMODULE)Lib,"PS2EgetLibType");
					PS2E_GetLibName    =  (_PS2EgetLibName) GetProcAddress((HMODULE)Lib,"PS2EgetLibName");
					PS2E_GetLibVersion =  (_PS2EgetLibVersion) GetProcAddress((HMODULE)Lib,"PS2EgetLibVersion");

		
         		if (PS2E_GetLibType != NULL && PS2E_GetLibName != NULL && PS2E_GetLibVersion != NULL) {
		         	unsigned long version = PS2E_GetLibVersion();
					long type = PS2E_GetLibType();
					sprintf(tmpStr, "%s %d.%d", PS2E_GetLibName(), (version>>8)&0xff, version&0xff);
								
				switch (type)
					{
                
                       case PS2E_LT_GS: // Gfx plugin 
                       {
                        ind = SendDlgItemMessage(hwnd,IDC_COMBO_GFX,CB_ADDSTRING,0,(LPARAM)tmpStr);
					    sprintf(GFXPLUGINS[ind],tmpStr2);
					    break;
					   }
					   
                       case PS2E_LT_PAD: // Input plugin
                       {
                        ind = SendDlgItemMessage(hwnd,IDC_COMBO_INPUT,CB_ADDSTRING,0,(LPARAM)tmpStr);
					    sprintf(INPUTPLUGINS[ind],tmpStr2);
					    break;
					   }
					   
					   case PS2E_LT_SPU2: // Sound plugin
                       {
                        ind = SendDlgItemMessage(hwnd,IDC_COMBO_SOUND,CB_ADDSTRING,0,(LPARAM)tmpStr);
					    sprintf(SOUNDPLUGINS[ind],tmpStr2);
					    break;
        			   }


					   case PS2E_LT_SIF: //Sif Plugin
					   {
						ind = SendDlgItemMessage(hwnd,IDC_COMBO_SIF,CB_ADDSTRING,0,(LPARAM)tmpStr);
						sprintf(SIFPLUGINS[ind],tmpStr2);
						break;
					   }
					   
					   case PS2E_LT_DBG: //
					   {
						ind = SendDlgItemMessage(hwnd,IDC_COMBO_DBG,CB_ADDSTRING,0,(LPARAM)tmpStr);
						sprintf(DBGPLUGINS[ind],tmpStr2);
						break;
					   }


				    } 
				   //FreeLibrary((HMODULE)Lib);
      			} // end of if   
				//FreeLibrary((HMODULE)Lib);
			} while (FindNextFile(Find,&FindData));

			ind = FindPluginToSelect(GSPluginToUse,PS2E_LT_GS);
			SendDlgItemMessage(hwnd, IDC_COMBO_GFX, CB_SETCURSEL, ind, 0);
	
	        ind = FindPluginToSelect(InputPluginToUse,PS2E_LT_PAD);
            SendDlgItemMessage(hwnd, IDC_COMBO_INPUT, CB_SETCURSEL, ind, 0);
            
            ind = FindPluginToSelect(SifPluginToUse,PS2E_LT_SIF);
			SendDlgItemMessage(hwnd, IDC_COMBO_SIF, CB_SETCURSEL, ind, 0);

            ind = FindPluginToSelect(DBGPluginToUse,PS2E_LT_DBG);
			SendDlgItemMessage(hwnd, IDC_COMBO_DBG, CB_SETCURSEL, ind, 0);

			
			TranslateConfigDialog(hwnd) ;

			if(bRunning == TRUE) {
				EnableWindow(GetDlgItem(hwnd, IDC_COMBO_GFX), FALSE);
                                EnableWindow(GetDlgItem(hwnd, IDC_COMBO_INPUT), FALSE);
                                EnableWindow(GetDlgItem(hwnd, IDC_COMBO_SIF), FALSE);
                                EnableWindow(GetDlgItem(hwnd, IDC_COMBO_DBG), FALSE);
                        }
                        else {
                                EnableWindow(GetDlgItem(hwnd, IDC_COMBO_GFX), TRUE);
                                EnableWindow(GetDlgItem(hwnd, IDC_COMBO_INPUT), TRUE);
                                EnableWindow(GetDlgItem(hwnd, IDC_COMBO_SIF), TRUE);
                                EnableWindow(GetDlgItem(hwnd, IDC_COMBO_DBG), TRUE);
                        }
			
    	return TRUE;
        }
        
         case WM_CLOSE:
			EndDialog(hwnd,0);
			break;
         
         case WM_COMMAND: 
            switch (HIWORD(wParam))
            {
              case CBN_SELCHANGE : 
                       switch (LOWORD(wParam)) 
                       {
                          case  IDC_COMBO_INPUT:
                                  ind = SendDlgItemMessage( hwnd, IDC_COMBO_INPUT, CB_GETCURSEL, 0, 0);
                                  WriteCfgString( "Plugins", "Input", INPUTPLUGINS[ind]);
                                  LoadInputPluggin( INPUTPLUGINS[ind]) ;                  
                          break;
                          case  IDC_COMBO_GFX:
                                  ind = SendDlgItemMessage(hwnd, IDC_COMBO_GFX, CB_GETCURSEL, 0, 0);
                                  WriteCfgString( "Plugins", "Video", GFXPLUGINS[ind]);
                                  LoadGsPluggin( GFXPLUGINS[ind] ) ;
                          break; 
                          case  IDC_COMBO_DBG:
                                  ind = SendDlgItemMessage(hwnd, IDC_COMBO_DBG, CB_GETCURSEL, 0, 0);
                                  WriteCfgString( "Plugins", "Dbg", DBGPLUGINS[ind]);
				                  LoadDEBUGPluggin( DBGPLUGINS[ind] );
                          break; 
                          case  IDC_COMBO_SIF: 
                                   ind = SendDlgItemMessage(hwnd, IDC_COMBO_SIF, CB_GETCURSEL, 0, 0);
                                   WriteCfgString( "Plugins", "Sif", SIFPLUGINS[ind]);                 
                          break; 
                          case IDC_COMBO_SOUND:
                          break;             
                       }
              break;        
            }
            switch (LOWORD(wParam)) 
            { 
			  case IDOK: 
				EndDialog( hwnd, 1);
				break;
            
               case IDGFXCONFIG:	GSconfigure();               break;                  
               case IDGFXTEST:      GStest();                    break;
               case IDGFXABOUT:     GSabout();  	             break;

               case IDINPUTCONFIG:  PAD1configure();             break;
               case IDINPUTTEST:	PAD1test();                  break;
               case IDINPUTABOUT:   PAD1about();                 break;
               
               case IDSOUNDCONFIG:          	break;
               case IDSOUNDTEST:               	break;
               case IDSOUNDABOUT:               break;
               
               case IDSIFCONFIG:	SIFDllConfig(hwnd);			break;
               case IDSIFTEST:      SIFDllTest(hwnd);           break;
               case IDSIFABOUT:     SIFDllAbout(hwnd);          break;
			}
			break;
	 } 
	 return FALSE;	
}


void OpenPluginsDialog(HWND hwnd) 
{
    DialogBox( App_hInstance, MAKEINTRESOURCE(IDD_CONFIG_PLUGINS), hwnd, (DLGPROC)ConfigureDlgProc);

}

//*******************************************************************
//*******************************************************************

void FindAnyPlugin( long searchType, char * Ret)
{
    _PS2EgetLibType    PS2E_GetLibType;
	_PS2EgetLibName    PS2E_GetLibName;
	_PS2EgetLibVersion PS2E_GetLibVersion;
	WIN32_FIND_DATA FindData ;
    HANDLE Find;
	HANDLE Lib;
    char tmpStr[256];
    
                strcpy(tmpStr, "plugins/");
				strcat(tmpStr, "*.dll");
				Find = FindFirstFile(tmpStr, &FindData);
				do{
				    sprintf(tmpStr,"%s%s", "plugins/", FindData.cFileName);
				    
				    Lib = LoadLibrary(tmpStr);
				    PS2E_GetLibType = (_PS2EgetLibType) GetProcAddress((HMODULE)Lib,"PS2EgetLibType");
					PS2E_GetLibName    =  (_PS2EgetLibName) GetProcAddress((HMODULE)Lib,"PS2EgetLibName");
					PS2E_GetLibVersion =  (_PS2EgetLibVersion) GetProcAddress((HMODULE)Lib,"PS2EgetLibVersion");

		
         		     if (PS2E_GetLibType != NULL && PS2E_GetLibName != NULL && PS2E_GetLibVersion != NULL) {
		         	     long type = PS2E_GetLibType();
		         	     if (type & searchType)
		         	      {
                                FreeLibrary((HMODULE)Lib);
                                sprintf(Ret,tmpStr);
                                break;
                          }
					     } // end of if   
				     FreeLibrary((HMODULE)Lib);
				
				} while (FindNextFile(Find,&FindData));

}


void LoadDefaultPlugins() 
{
    ReadCfgString("Plugins","Video","",GSPluginToUse);
    if (strcmp(GSPluginToUse,"")==0)
    {
        FindAnyPlugin(PS2E_LT_GS,GSPluginToUse) ;    
    }
    
	ReadCfgString("Plugins","Input","",InputPluginToUse);
	if (strcmp(InputPluginToUse,"")==0)
	{
	    FindAnyPlugin(PS2E_LT_PAD,InputPluginToUse) ;
    }
    ReadCfgString("Plugins","SIF","plugins/SifPlugin.dll",SifPluginToUse) ;

	ReadCfgString("Plugins","Dbg","",DBGPluginToUse);
	if (strcmp(DBGPluginToUse,"")==0)
	{
	    FindAnyPlugin(PS2E_LT_DBG,DBGPluginToUse) ;
    }




}    
