/**
 * nSX2 - an OpenSource PS2 Emulator
 * Copyright (C) 2002 Muad
 *
 * homepage: http://nsx2.emulation64.com
 * email address: atreides.muaddib@caramail.com
 * 
 * If you want to contribute to the project please contact
 * me first (maybe someone is already making what you are
 * planning to do).
 *
 *
 * This program is free software; you can redistribute it and/
 * or modify it under the terms of the GNU General Public Li-
 * cence as published by the Free Software Foundation; either
 * version 2 of the Licence, or any later version.
 *
 * This program is distributed in the hope that it will be use-
 * ful, but WITHOUT ANY WARRANTY; without even the implied war-
 * ranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public Licence for more details.
 *
 * You should have received a copy of the GNU General Public
 * Licence along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
**/
#include <stdio.h>
#include <stdlib.h>

#include "nSX2.h"
#include "PS2Edefs.h"


/* create a function to avoid this kind of include */ 
#include "timers.h"




/***********************************************************************************************/

void	MipsMemoryWrite ( _u128 fVal  , _u8 fDataType , _u64 fBase , int fOffset ) {
	
	_u8	Mem_Perform;
	_u64	PhysicalAdd;
	
	_u8 * MemTarget;

	char tempText[100];	
	
	Mem_Perform = 0;
	PhysicalAdd = ( fBase + fOffset ) & 0xFFFFFFFF;



	if ( (PhysicalAdd >= (_u32)0x11000000) && (PhysicalAdd < (_u32)0x11fffff0) ) {
		EMU_LOG("[ VU0 & VU1 MEMORY : WRITE ACCESS DETECTED ]\n");
	}

	//
	if ( (( PhysicalAdd >= (_u32)0x70000000 ) && ( PhysicalAdd <= (_u32)0x70003FFF )) 
	)	{

		MemTarget = SPRMemory;		
		MipsSPRMemWrite( MemTarget , fVal ,  fDataType , fBase , fOffset );
		Mem_Perform = 1;
		
	}


	//	0x10000000 - 0x1000FFFF	
	if ( ( PhysicalAdd >= 0x10000000 ) && ( PhysicalAdd <= 0x1000FFFF ) ){
		MemTarget = SpecialAreaMemory;
		MipsSpecialMemWrite( MemTarget , fVal ,  fDataType , fBase , fOffset );
		Mem_Perform = 1;
	}


	//	0x12000000 - 0x12001080	
	if ( ( PhysicalAdd >= 0x12000000 ) && ( PhysicalAdd <= 0x12001080 ) ){
		MemTarget = GSAreaMemory;
		MipsGSAreaMemWrite( MemTarget, fVal ,  fDataType , fBase , fOffset );
		Mem_Perform = 1;
	}


	// 0x1FC00000 - 0x1FFFFFFF	ROM BIOS Mirror Adress.	(not cached)
	// 0x9FC00000 - 0x9FFFFFFF	ROM BIOS Mirror Adress.	(cached)
	// 0xBFC00000 - 0xBFFFFFFF	ROM BIOS Mirror Adress.	(not cached)
	if ( 	(( PhysicalAdd >= 0x1FC00000 ) && ( PhysicalAdd < 0x1FFFFFFF )) ||
	 		(( PhysicalAdd >= 0x9FC00000 ) && ( PhysicalAdd < 0x9FFFFFFF )) ||
	 		(( PhysicalAdd >= 0xBFC00000 ) && ( PhysicalAdd < 0xBFFFFFFF )) )
	 {

		MemTarget = BiosMemory; 
	 	MipsBiosAreaMemWrite( MemTarget, fVal ,  fDataType , fBase , fOffset );
		Mem_Perform = 1;
	 }	


	if ( Mem_Perform == 0) {

		PhysicalAdd = PhysicalAdd &	0x01FFFFFF;

		MemTarget = MipsMemory;
		
		switch ( fDataType ) {
			case DATA_QUAD:		MipsMemoryWriteDW 	( MemTarget, fVal.lo, PhysicalAdd );
								MipsMemoryWriteDW 	( MemTarget, fVal.hi, PhysicalAdd + 8 );break;

			case DATA_DW:		MipsMemoryWriteDW	( MemTarget, fVal.lo, PhysicalAdd );break;

			case DATA_W:		MipsMemoryWriteW	( MemTarget, fVal.lo, PhysicalAdd );break;

			case DATA_HW:		MipsMemoryWriteHW	( MemTarget, fVal.lo, PhysicalAdd );break;

			case DATA_B:		MipsMemoryWriteB	( MemTarget, fVal.lo, PhysicalAdd );break;
		
			default:			
#ifdef EMU_LOG
				EMU_LOG("Mem:Write DataType Not Handled\n");
#endif						
				MipsCpu.halt = 1;break;
		}
		Mem_Perform = 1;
	
	}

}
/*
To DO:
Maybe Change this to #define

*/
void MipsMemoryWriteDW ( _u8 * Mem ,  _u64 fVal , _u64 fAdd ) {
	(_u64)*(( _u64 *)(Mem + fAdd )) = (_u64)fVal;
}

void MipsMemoryWriteW  (_u8 * Mem ,  _u64 fVal , _u64 fAdd ) {
	(_u32)*(( _u32 *)(Mem + fAdd)) = (_u32)(fVal & 0xFFFFFFFF);	
}

void MipsMemoryWriteHW  ( _u8 * Mem , _u64 fVal , _u64 fAdd ) {
	(_u16)*(( _u16 *)(Mem + fAdd)) = (_u16)(fVal & 0xFFFF);
}

void MipsMemoryWriteB  ( _u8 * Mem ,_u64 fVal , _u64 fAdd ) {
	(_u8)*(Mem + fAdd) = (_u8)(fVal & 0xFF);
}

/******************************************************************************************/

void	MipsMemoryRead ( _u128 * fVal  , _u8 fDataType , _u64 fBase , int fOffset ) {
	char tempText[100];	

	_u8	Mem_Perform;
	_u64	PhysicalAdd;
	
	_u8 * MemTarget;
	
	Mem_Perform = 0;
	PhysicalAdd = ( fBase + fOffset ) & 0xFFFFFFFF;
/*
<expert-> go map KSEG0 after 0x82000000
*/

	if ( (PhysicalAdd >= (_u32)0x11000000) && (PhysicalAdd < 0x11fffff0) ) {
		EMU_LOG("[ VU0 & VU1 MEMORY : READ ACCESS DETECTED ]\n");

	}



	if ( (( PhysicalAdd >= 0x70000000 ) && ( PhysicalAdd <= 0x70003FFF )) ||
		 (( PhysicalAdd >= 0xF0000000 ) && ( PhysicalAdd <= 0xF0003FFF )) ||
		 (( PhysicalAdd >= 0x80000000 ) && ( PhysicalAdd <= 0x80003FFF )) 
		){
		MemTarget = SPRMemory;		
		MipsSPRMemRead( MemTarget , fVal ,  fDataType , fBase , fOffset );
		Mem_Perform = 1;
		
	}

	
	
	//	0x10000000 - 0x1000FFFF	
	if ( ( PhysicalAdd >= 0x10000000 ) && ( PhysicalAdd <= 0x1000FFFF ) ){
		MemTarget = SpecialAreaMemory;
		MipsSpecialMemRead(MemTarget , fVal ,  fDataType , fBase , fOffset );
		Mem_Perform = 1;
		
	}

	//	0x12000000 - 0x12001080	
	if ( ( PhysicalAdd >= 0x12000000 ) && ( PhysicalAdd <= 0x12001080 ) ){
		MemTarget = GSAreaMemory;
		MipsGSAreaMemRead( MemTarget,  fVal ,  fDataType , fBase , fOffset );
		Mem_Perform = 1;
		
	}



	if ( Mem_Perform == 0 ) {

		PhysicalAdd = PhysicalAdd &	0x01FFFFFF;
		MemTarget = MipsMemory; 
		switch ( fDataType ) {
		
			case DATA_QUAD:		MipsMemoryReadDQ 	( MemTarget, fVal, PhysicalAdd );break;
			
			case DATA_DW:		MipsMemoryReadDW	( MemTarget, fVal, PhysicalAdd );break;
			
			case DATA_W:		MipsMemoryReadW		( MemTarget, fVal, PhysicalAdd );break;

			case DATA_HW:		MipsMemoryReadHW	( MemTarget, fVal, PhysicalAdd );break;

			case DATA_B:		MipsMemoryReadB		( MemTarget, fVal, PhysicalAdd );break;

			default:			
#ifdef EMU_LOG
				EMU_LOG("Mem:Read DataType Not Handled\n");
#endif		
				MipsCpu.halt = 1;break;
		}
		Mem_Perform = 1;
	
	}

}

/*
TO DO 
Change this to Macro

*/
void MipsMemoryReadDQ ( _u8 * Mem,  _u128 * fVal , _u64 fAddr ) {
	fVal->lo = (_u64)*(( _u64 *)((Mem + fAddr )));
	fVal->hi = (_u64)*(( _u64 *)((Mem + (fAddr  + 8 ))));
}

void MipsMemoryReadDW ( _u8 * Mem,  _u128 * fVal , _u64 fAddr ) {
	fVal->lo = (_u64)*(( _u64 *)(Mem + fAddr ));
}

void MipsMemoryReadW ( _u8 * Mem, _u128 * fVal , _u64 fAddr ) {
	fVal->lo = (_u32)*(( _u32 *)(Mem + fAddr));
}

void MipsMemoryReadHW ( _u8 * Mem,  _u128 * fVal , _u64 fAddr ) {
	fVal->lo = (_u16)*(( _u16 *)(Mem + fAddr));
}


void MipsMemoryReadB ( _u8 * Mem,  _u128 * fVal , _u64 fAddr ) {
	fVal->lo =(_u8)*(Mem + fAddr);
}


/*****************************************************************************************************/

void  MipsSpecialMemRead( _u8 * Mem, _u128 * fVal  , _u8 fDataType , _u64 fBase , int fOffset ) {
	char tempText[100];	
	_u64 	RealAdd;
	_u64	PhysicalAddress;

	
	

	PhysicalAddress = fBase + fOffset;
	RealAdd 		= PhysicalAddress & 0xFFFF;


	if ( (RealAdd >= 0x0000) && (RealAdd < 0x1ff0) ) {

		TIMERS_Read ( PhysicalAddress , fVal , fDataType );
	}

	if ( (RealAdd >= 0x2000) && (RealAdd < 0x2ff0) ) {

		switch ( fDataType ) {
	
			case DATA_QUAD:	MipsMemoryReadDQ 	( Mem, fVal, RealAdd );break;
					
			case DATA_DW:	MipsMemoryReadDW	( Mem, fVal, RealAdd );break;

		   case DATA_HW:	MipsMemoryReadHW	( Mem, fVal, RealAdd );break;
		
			case DATA_W:	MipsMemoryReadW 	( Mem, fVal, RealAdd );break;

			case DATA_B:	MipsMemoryReadB		( Mem, fVal, RealAdd );break;

			default: 		

#ifdef EMU_LOG
	EMU_LOG("MemSpecial:Read DataType Not Handled\n");
#endif		



							MipsCpu.halt = 1;break;
		}

#ifdef EMU_LOG
	EMU_LOG("[ IPU AREA : READ ACCESS DETECTED ]\n");
#endif	

		
	}

	if ( (RealAdd >= 0x3000) && (RealAdd < 0x37f0) ) {
		GIF_Read ( PhysicalAddress , fVal , fDataType );
	}

	if ( (RealAdd >= 0x3800) && (RealAdd < 0x3bf0) ) {
		VIF0_Read ( PhysicalAddress , fVal , fDataType );
	}

	if ( (RealAdd >= 0x3c00) && (RealAdd < 0x3ff0) ) {
		VIF1_Read ( PhysicalAddress , fVal , fDataType );
	}


	if ( (RealAdd >= 0x8000) && (RealAdd < 0xF000) ) {
		DMAC_Read ( PhysicalAddress , fVal , fDataType );
	}

	if ( RealAdd >= 0xF000 ) {

		INTC_Read ( PhysicalAddress , fVal , fDataType );

	}
}


void  MipsSpecialMemWrite( _u8 * Mem , _u128  fVal  , _u8 fDataType , _u64 fBase , int fOffset ) {
	char tempText[100];	

	_u64 	RealAdd;
	_u64	PhysicalAddress;

	
	

	PhysicalAddress = fBase + fOffset;
	RealAdd 		= PhysicalAddress & 0xFFFF;


	if ( (RealAdd >= 0x0000) && (RealAdd < 0x1ff0) ) {

		TIMERS_Write ( PhysicalAddress , fVal , fDataType );
	}

	if ( (RealAdd >= 0x2000) && (RealAdd < 0x2ff0) ) {

		switch ( fDataType ) {

			case DATA_QUAD:		MipsMemoryWriteDW 	( Mem, fVal.lo, RealAdd );
								MipsMemoryWriteDW 	( Mem, fVal.hi, RealAdd + 8 );break;

				
			case DATA_DW:	MipsMemoryWriteDW	( Mem, fVal.lo, RealAdd );break;
				
			case DATA_W:	MipsMemoryWriteW	( Mem, fVal.lo, RealAdd );break;

			case DATA_HW:	MipsMemoryWriteHW	( Mem, fVal.lo, RealAdd );break;

			case DATA_B:	MipsMemoryWriteB	( Mem, fVal.lo, RealAdd );break;
										
			default:		

#ifdef EMU_LOG
	EMU_LOG("MemSpecial:Write DataType Not Handled\n");
#endif	


						MipsCpu.halt = 1;break;
		}

#ifdef EMU_LOG
	EMU_LOG("[ IPU AREA : WRITE ACCESS DETECTED\n");
#endif	
	
	}


	if ( (RealAdd >= 0x3000) && (RealAdd < 0x37f0) ) {
		GIF_Write ( PhysicalAddress , fVal , fDataType );
	}

	if ( (RealAdd >= 0x3800) && (RealAdd < 0x3bf0) ) {
		VIF0_Write ( PhysicalAddress , fVal , fDataType );
	}

	if ( (RealAdd >= 0x3c00) && (RealAdd < 0x3ff0) ) {
		VIF1_Write ( PhysicalAddress , fVal , fDataType );
	}

	if ( (RealAdd >= 0x8000) && (RealAdd < 0xF000) ) {
		DMAC_Write ( PhysicalAddress , fVal , fDataType );
	}

	if ( RealAdd >= 0xF000 ) {
		INTC_Write ( PhysicalAddress , fVal , fDataType );
	}


}



/***********************************************************************************************/



void  MipsGSAreaMemWrite( _u8 * Mem , _u128  fVal  , _u8 fDataType , _u64 fBase , int fOffset ) {
	char tempText[100];	
	_u32 Add;

	Add = ( fBase + fOffset ) ;
	
	switch ( fDataType ) {
			
		case DATA_DW:	(*GSwrite64)	( Add, fVal.lo );	break;

		case DATA_W:	(*GSwrite32)	( Add, (_u32) (fVal.lo&0x00000000FFFFFFFF) );	break;
			
		default:		
#ifdef EMU_LOG
			EMU_LOG("MemGSArea:Write DataType Not Handled\n");
#endif
			MipsCpu.halt = 1;break;
	}
}



void MipsGSAreaMemRead( _u8 * Mem, _u128 * fVal  , _u8 fDataType , _u64 fBase , int fOffset ) {
	char tempText[100];	
	_u64 RealAdd;
	RealAdd = ( fBase + fOffset ) ; 


	switch ( fDataType ) {
		
		case DATA_DW:	fVal->lo = (*GSread64)((_u32)RealAdd) ;	break;
		case DATA_W:	fVal->lo = (*GSread32)((_u32)RealAdd) ;	break;
	
		default:		
#ifdef EMU_LOG
			EMU_LOG("MemGSArea:Read DataType Not Handled\n");
#endif
			MipsCpu.halt = 1;break;
	}


	/* Timer GS VSync */
	if ( RealAdd == 0x12001000 ) {

		if ( GS_VSync_Wait == 0 ) {

			if ( fDataType == DATA_DW ) fVal->lo = fVal->lo  | 0x00032000;
   			fVal->lo = fVal->lo | 0xF;


		} else {

   			if ( (Timer[ NSX2_TIMER_GS ].Reg->hold > 230 ) ) {		
 
				if ( fDataType == DATA_DW ) fVal->lo = fVal->lo  | 0x00032000;
   		
   				fVal->lo = fVal->lo | 0xF;
 				Ps2Bios.NeedUpdateScreen = TRUE;
	
 	//  		Ps2Bios.AccesToGS = TRUE;
				GSUpdate();

				Timer[ NSX2_TIMER_GS ].Reg->hold = 0;
	//			CLEAR_MODE_OVFF_TIMER( NSX2_TIMER_GS );
 			
 			}	

 		} 

	}

}

/*****************************************************************************************************/


void  MipsBiosAreaMemRead(_u8 * Mem, _u128 * fVal  , _u8 fDataType , _u64 fBase , int fOffset ) {

	_u64 RealAdd;
	RealAdd = ( fBase + fOffset ) & 0x3FFFFF;

	switch ( fDataType ) {

		case DATA_W:	MipsMemoryReadW	( Mem, fVal, RealAdd );break;

		case DATA_B:	MipsMemoryReadB ( Mem, fVal, RealAdd );break;
												
		default:		
#ifdef EMU_LOG
			EMU_LOG("MemBIOS:Read DataType Not Handled\n");
#endif
			MipsCpu.halt = 1;break;
	}

}

void  MipsBiosAreaMemWrite( _u8 * Mem , _u128  fVal  , _u8 fDataType , _u64 fBase , int fOffset ) {

	_u64 RealAdd;
	RealAdd = ( fBase + fOffset ) & 0x3FFFFF;

	switch ( fDataType ) {
				
		case DATA_DW:	MipsMemoryWriteDW	( Mem, fVal.lo, RealAdd );break;
				
		case DATA_W:	MipsMemoryWriteW	( Mem, fVal.lo, RealAdd );break;
										
		default:		
#ifdef EMU_LOG
			EMU_LOG("MemBios:Write DataType Not Handled\n");
#endif
			MipsCpu.halt = 1;break;
	}

}

/*****************************************************************************************************/

void  MipsSPRMemRead( _u8 * Mem, _u128 * fVal  , _u8 fDataType , _u64 fBase , int fOffset ) {
	char tempText[100];	
	_u64 RealAdd;
	RealAdd = ( fBase + fOffset ) & 0x3FFF;

	switch ( fDataType ) {

		case DATA_QUAD:	MipsMemoryReadDQ 	( Mem, fVal, RealAdd );break;
					
		case DATA_DW:	MipsMemoryReadDW	( Mem, fVal, RealAdd );break;

		case DATA_HW:	MipsMemoryReadHW	( Mem, fVal, RealAdd );break;
		
		case DATA_W:	MipsMemoryReadW 	( Mem, fVal, RealAdd );break;

		case DATA_B:	MipsMemoryReadB		( Mem, fVal, RealAdd );break;

		default: 	

#ifdef EMU_LOG
			EMU_LOG("MemSPR:Read DataType Not Handled\n");
#endif
			MipsCpu.halt = 1;break;
	}



}


void  MipsSPRMemWrite( _u8 * Mem , _u128  fVal  , _u8 fDataType , _u64 fBase , int fOffset ) {
	char tempText[100];	
	_u64 RealAdd;
	RealAdd = ( fBase + fOffset ) & 0x3FFF;

	switch ( fDataType ) {

		case DATA_QUAD:		MipsMemoryWriteDW 	( Mem, fVal.lo, RealAdd );
							MipsMemoryWriteDW 	( Mem, fVal.hi, RealAdd + 8 );break;

				
		case DATA_DW:	MipsMemoryWriteDW	( Mem, fVal.lo, RealAdd );break;
				
		case DATA_W:	MipsMemoryWriteW	( Mem, fVal.lo, RealAdd );break;

		case DATA_HW:	MipsMemoryWriteHW	( Mem, fVal.lo, RealAdd );break;

		case DATA_B:	MipsMemoryWriteB	( Mem, fVal.lo, RealAdd );break;
										
		default:	

#ifdef EMU_LOG
			EMU_LOG("MemSPR:Write DataType Not Handled\n");
#endif
			MipsCpu.halt = 1;break;
	}




}

