/**
 * nSX2 - an OpenSource PS2 Emulator
 * Copyright (C) 2002 Muad
 *
 * homepage: http://nsx2.emulation64.com
 * email address: atreides.muaddib@caramail.com
 * 
 * If you want to contribute to the project please contact
 * me first (maybe someone is already making what you are
 * planning to do).
 *
 *
 * This program is free software; you can redistribute it and/
 * or modify it under the terms of the GNU General Public Li-
 * cence as published by the Free Software Foundation; either
 * version 2 of the Licence, or any later version.
 *
 * This program is distributed in the hope that it will be use-
 * ful, but WITHOUT ANY WARRANTY; without even the implied war-
 * ranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public Licence for more details.
 *
 * You should have received a copy of the GNU General Public
 * Licence along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
**/
#include <stdlib.h>
#include <stdio.h>

#include "nSX2.h"
#include "PS2Edefs.h"

#include "timers.h"


/* Update the Timer GS */
void TIMERS_Update_GS ( void ) {

	_u8		boucle;
	_u64	ElapsedCycles;

	/* Get the number of cycles since last call */
	ElapsedCycles = MipsCpu.Nb_Cycles - TIMERS_PreviousCycles;

	/* Update the Count Value */
	Timer[ NSX2_TIMER_GS ].Reg->count = (_u32) ( Timer[ NSX2_TIMER_GS ].Reg->count + (_u32) ElapsedCycles ) ;

	/* if the count > Compare Value */
	if ( Timer[ NSX2_TIMER_GS ].Reg->count > Timer[ NSX2_TIMER_GS ].Reg->comp )  {

		// Set the Overflow , for Reading GS_Synx by the EE Core */
		SET_MODE_OVFF_TIMER( NSX2_TIMER_GS );

		// if a H-Sync occurs 
		if ( GET_MODE_OVFF_TIMER( NSX2_TIMER_GS ) == 0x1 ) {
			

		 	// For all the Timers... 
			// I check if the PS2 Timers are syncronised by H-Sync 
			for ( boucle = 0 ; boucle < 4 ; boucle ++ ) {
				// If the Timer is running  
				if ( GET_MODE_CUE_TIMER( boucle ) == 0x1 ) {

					// And the Update mode is the H sync 
					if ( GET_MODE_CLKS_TIMER( boucle ) == 0x3 ) {
						// Update the count value of the Timer 
						Timer[ boucle ].Reg->count 	= Timer[ boucle ].Reg->count + 1;

						// Performs Test on Overflow,Equal, ZEroReturn 
						TIMERS_TestOverlow 		( boucle );
						TIMERS_TestEqual 		( boucle );
						TIMERS_TestZeroReturn 	( boucle );

					}//End of H-sync Mode Test

				}//End of Running Test
			}//End of Loop
	


			// Reset the Count Value for the next 
			Timer[ NSX2_TIMER_GS ].Reg->count = 0;

			// I use the hold Value to count the number of H-Sync 
			Timer[ NSX2_TIMER_GS ].Reg->hold = Timer[ NSX2_TIMER_GS ].Reg->hold + 1;



			// if the number of H-Sync is > 240 ; it's a V-Sync
			if ( Timer[ NSX2_TIMER_GS ].Reg->hold > 240 ) {

/*				
				if( !Ps2Bios.AccesToGS ) {
*/					Ps2Bios.NeedUpdateScreen 	= TRUE;						
					GSUpdate();
					//PrintToEmuOutput("VSync by GS Timer ");
			
						

					
				//}

			// Reset the Number of H-Sync 

			Timer[ NSX2_TIMER_GS ].Reg->hold = 0;

			CLEAR_MODE_OVFF_TIMER( NSX2_TIMER_GS );		

		}// end of if the number of H-Sync is > 240 
				
		}/* end of if a H-Sync occurs */

	}/* end of if the count > Compare Value */
}



/* Update the Timer Input */
void TIMERS_Update_Input ( void ) {

	_u8		boucle;
	_u64	ElapsedCycles;


	/* Get the number of cycles since last call */	
	ElapsedCycles = MipsCpu.Nb_Cycles - TIMERS_PreviousCycles;
		
	/* Update the Count Value */
	Timer[ NSX2_TIMER_INPUT ].Reg->count = (_u32) ( Timer[ NSX2_TIMER_INPUT ].Reg->count + (_u32) ElapsedCycles ) ;

	/* if the count > Compare Value */
	if ( Timer[ NSX2_TIMER_INPUT ].Reg->count > Timer[ NSX2_TIMER_INPUT ].Reg->comp )  {
		Ps2Bios.NeedUpdatePad 					= TRUE;
		Timer[ NSX2_TIMER_INPUT ].Reg->count 	= 0;
		PadAndKeyUpdate();
	}


}

/* Update the Timer Thread */
void TIMERS_Update_Thread ( void ) {

	_u8		boucle;
	_u64	ElapsedCycles;


	/* Get the number of cycles since last call */	
	ElapsedCycles = MipsCpu.Nb_Cycles - TIMERS_PreviousCycles;
		
	/* Update the Count Value */
	Timer[ NSX2_THREAD_TIMER ].Reg->count = (_u32) ( Timer[ NSX2_THREAD_TIMER ].Reg->count + (_u32) ElapsedCycles ) ;

	/* if the count > Compare Value */
	if ( Timer[ NSX2_THREAD_TIMER ].Reg->count > Timer[ NSX2_THREAD_TIMER ].Reg->comp )  {
		UpdatePS2Scheduler();
		Timer[ NSX2_THREAD_TIMER ].Reg->count = 0;

	}

}



