/**
 * nSX2 - an OpenSource PS2 Emulator
 * Copyright (C) 2002 Muad
 *
 * homepage: http://nsx2.emulation64.com
 * email address: atreides.muaddib@caramail.com
 * 
 * If you want to contribute to the project please contact
 * me first (maybe someone is already making what you are
 * planning to do).
 *
 *
 * This program is free software; you can redistribute it and/
 * or modify it under the terms of the GNU General Public Li-
 * cence as published by the Free Software Foundation; either
 * version 2 of the Licence, or any later version.
 *
 * This program is distributed in the hope that it will be use-
 * ful, but WITHOUT ANY WARRANTY; without even the implied war-
 * ranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public Licence for more details.
 *
 * You should have received a copy of the GNU General Public
 * Licence along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
**/

#include <stdlib.h>

#include <windows.h>
#define _WIN32_IE 0x0500
#include <commctrl.h>
#include "resource.h" 

#define LangFileName "nSX2.lng"
#define DefaultLangFileName "/Lang/english.lng"

static char LangFile[_MAX_PATH];
static int LastLang = -1;
static char ApplicationPath[_MAX_PATH];

void InitLangFileName(void) {
	char path_buffer[_MAX_PATH], drive[_MAX_DRIVE] ,dir[_MAX_DIR];
	char fname[_MAX_FNAME],ext[_MAX_EXT];
	
	GetModuleFileName(NULL,path_buffer,sizeof(path_buffer));
	_splitpath( path_buffer, drive, dir, fname, ext );
	sprintf(LangFile,"%s%s%s",drive,dir,LangFileName);
	sprintf(ApplicationPath,"%s%s",drive,dir);
} 


void Translate(char* GuiWord,char* Ret)
{
   char Language[100];
   char Path[_MAX_PATH];
   char Path2[_MAX_PATH];
   GetPrivateProfileString("Default","Language","English",Language,sizeof(Language),LangFile);
   GetPrivateProfileString(Language,"Path","Lang/english.lng",Path,sizeof(Path),LangFile);
   sprintf(Path2,"%s/%s",ApplicationPath,Path);
   GetPrivateProfileString(Language,GuiWord,GuiWord,Ret,100,Path2);
}


void TranslateDefault(char* GuiWord,char *Default,char* Ret)
{
   char Language[100];
   char Path[_MAX_PATH];
   char Path2[_MAX_PATH];
   GetPrivateProfileString("Default","Language","English",Language,sizeof(Language),LangFile);
   GetPrivateProfileString(Language,"Path","Lang/english.lng",Path,sizeof(Path),LangFile);
   sprintf(Path2,"%s/%s",ApplicationPath,Path);
   GetPrivateProfileString(Language,GuiWord,Default,Ret,100,Path2); 
}


void SelectLang(HWND hWnd, int LangMenuID) {
	char String[800];
	HMENU hMenu = GetMenu(hWnd);
	MENUITEMINFO menuinfo;
	

	menuinfo.cbSize = sizeof(MENUITEMINFO);
	menuinfo.fMask = MIIM_TYPE;
	menuinfo.fType = MFT_STRING;
	menuinfo.dwTypeData = String;
	menuinfo.cch = sizeof(String);
	GetMenuItemInfo(hMenu,LangMenuID,FALSE,&menuinfo);

	WritePrivateProfileString("Default","Language",String,LangFile);
	if (LastLang != -1) {
		CheckMenuItem( hMenu, LastLang, MF_BYCOMMAND | MFS_UNCHECKED );
	}
	LastLang = LangMenuID;
	CheckMenuItem( hMenu, LastLang, MF_BYCOMMAND | MFS_CHECKED );
}


void SetupLanguages ( HMENU LangMenu ) {
    char String[800], Language[100], *RP;
	MENUITEMINFO menuinfo;
	int count,i;
    InitLangFileName();
    GetPrivateProfileString("Default","Language","English",Language,sizeof(Language),LangFile);
	menuinfo.cbSize = sizeof(MENUITEMINFO);
	menuinfo.fMask = MIIM_TYPE|MIIM_ID;
	menuinfo.fType = MFT_STRING;
	menuinfo.fState = MFS_ENABLED;
	menuinfo.dwTypeData = String;
	menuinfo.cch = sizeof(LangMenu);

  	memset(String,0,sizeof(String));
	GetPrivateProfileSectionNames(String,sizeof(String),LangFile);
	for (count = 1,RP = String; *RP != 0; RP += strlen(RP) + 1) {
		
        if (strcmp(RP,"Default") == 0) { continue; }
		if (strcmp(RP,"English") == 0) { 
		    if (strcmp(Language, "English") == 0) {
		  		    CheckMenuItem(LangMenu, ID_LANG_ENGLISH, MF_BYCOMMAND | MFS_CHECKED );
		            LastLang = ID_LANG_ENGLISH;
        		}
            continue; 
        }
		menuinfo.dwTypeData = RP;
		menuinfo.cch = strlen(RP) + 1;
		menuinfo.wID = ID_LANG_ENGLISH + count;
		InsertMenuItem(LangMenu, count++, TRUE, &menuinfo);
		if (strcmp(Language, RP) == 0) {
		  CheckMenuItem(LangMenu, menuinfo.wID, MF_BYCOMMAND | MFS_CHECKED );
		  LastLang = menuinfo.wID;
        }
	}
}	


void SetItemTranslatedString(HWND hwnd,int ElementID,char* Str)
{
    char String[800];
    Translate(Str,String);  
    SetDlgItemText( hwnd, ElementID, String ); 
}


void SetItemTranslatedStringDefault(HWND hwnd,int ElementID,char* Str,char*Def)
{
    char String[800];
    TranslateDefault(Str,Def,String);
    SetDlgItemText( hwnd, ElementID, String );
}

void SetStatusTranslatedString(HWND hStatus,int section,char* Str)
{
    char String[800];
    Translate(Str,String);  
    SendMessage( hStatus, SB_SETTEXT, section, (LPARAM)String ); 
}

void SetMenuTranslatedString(HMENU hMenu,int elementID,char* Str)
{
    char Language[100], String[800];
    MENUITEMINFO menuinfo;
    Translate(Str,String);
            memset(&menuinfo, 0, sizeof(MENUITEMINFO));
            menuinfo.cbSize = sizeof(MENUITEMINFO);
            menuinfo.fMask = MIIM_TYPE;
	        menuinfo.fType = MFT_STRING;
	        menuinfo.dwTypeData = String;
	        menuinfo.cch = sizeof(String);
            SetMenuItemInfo(hMenu,elementID,TRUE,&menuinfo);
}


void SetWindowTranslatedTitle(HWND hwnd,char* str)
{
    char tmp[200] ;
    Translate(str,tmp);
    SetWindowText(hwnd,tmp);
}

void TranslateMenu(HMENU hMenu,HWND mainHWND)
{
     char Language[100], String[800];
	MENUITEMINFO menuinfo;
    int	menuCount,i;
    HMENU submenu;
    //Main menu
    SetMenuTranslatedString(hMenu,0,"File");
    SetMenuTranslatedString(hMenu,1,"Emulation");
    SetMenuTranslatedString(hMenu,2,"Debug");
    SetMenuTranslatedString(hMenu,3,"Settings_Menu");
    SetMenuTranslatedString(hMenu,4,"Help");
    
    //File menu
    submenu = GetSubMenu(hMenu,0) ;
    SetMenuTranslatedString(submenu,0,"Run CDVD");
    SetMenuTranslatedString(submenu,1,"Run ELF");
    SetMenuTranslatedString(submenu,5,"Exit");
    
    //Emulation menu
    submenu = GetSubMenu(hMenu,1) ;
    SetMenuTranslatedString(submenu,0,"Go");
    SetMenuTranslatedString(submenu,1,"Pause");
    SetMenuTranslatedString(submenu,2,"Reset");
    SetMenuTranslatedString(submenu,3,"Stop");
    
    //Debug menu
    submenu = GetSubMenu(hMenu,2) ;
    SetMenuTranslatedString(submenu,0,"Init");
    SetMenuTranslatedString(submenu,1,"View");
    SetMenuTranslatedString(submenu,2,"Close");
            
    //Settings menu
    submenu = GetSubMenu(hMenu,3) ;
    SetMenuTranslatedString(submenu,0,"Log");
    SetMenuTranslatedString(submenu,1,"Clear Log");
    SetMenuTranslatedString(submenu,2,"Plugins");
    SetMenuTranslatedString(submenu,4,"Emulator");
/*
    if (extLogger) { //  Not found external log dll,or failed to load it 
        EnableMenuItem(submenu, ID_SETTING_CLEAR_LOG,  MF_GRAYED );
    }
*/        
    //About menu
    submenu = GetSubMenu(hMenu,4) ;
    SetMenuTranslatedString(submenu,0,"About");
    SetMenuTranslatedString(submenu,1,"nSX2");
        
    DrawMenuBar(mainHWND);

}

void TranslateLogSheet(HWND hwnd)
{
    
    SetItemTranslatedString( hwnd , IDC_LOG_OPTIONS , "Log Options") ; 
    SetItemTranslatedString( hwnd , IDC_LOG_GS , "GS") ; 
    SetItemTranslatedString( hwnd , IDC_LOG_DMA , "DMA") ; 
    SetItemTranslatedString( hwnd , IDC_LOG_SYSCALL , "SYSCALL") ; 
    SetItemTranslatedString( hwnd , IDC_LOG_GIF , "GIF") ; 
    SetItemTranslatedString( hwnd , IDC_LOG_INTC , "INTC") ; 
    SetItemTranslatedString( hwnd , IDC_LOG_TIMER , "TIMER") ; 
    SetItemTranslatedString( hwnd , IDC_LOG_VIF , "VIF") ;
    SetItemTranslatedString( hwnd , IDC_LOG_THREAD , "THREAD") ;
    SetItemTranslatedString( hwnd , IDC_LOG_FILE , "FILE_settings_window") ;  
    
    SetWindowTranslatedTitle( hwnd, "Emulator Settings") ; 
}

void TranslateConfigDialog(HWND hwnd)
{
    char tmp[200] ;
    
    SetItemTranslatedString( hwnd , IDC_GFXPLUGIN , "GFX plugin") ;
    SetItemTranslatedString( hwnd , IDGFXCONFIG , "Config") ;
    SetItemTranslatedString( hwnd , IDGFXTEST , "Test") ;
    SetItemTranslatedString( hwnd , IDGFXABOUT , "About") ;
    
    SetItemTranslatedString( hwnd , IDC_SOUNDPLUGIN , "Sound plugin") ;
    SetItemTranslatedString( hwnd , IDSOUNDCONFIG , "Config") ;
    SetItemTranslatedString( hwnd , IDSOUNDTEST , "Test") ;
    SetItemTranslatedString( hwnd , IDSOUNDABOUT , "About") ;
    
    SetItemTranslatedString( hwnd , IDC_INPUTPLUGIN , "Input plugin") ;
    SetItemTranslatedString( hwnd , IDINPUTCONFIG , "Config") ;
    SetItemTranslatedString( hwnd , IDINPUTTEST , "Test") ;
    SetItemTranslatedString( hwnd , IDINPUTABOUT , "About") ;
    
    SetItemTranslatedString( hwnd , IDC_SIFPLUGIN , "SIF plugin") ;
    SetItemTranslatedString( hwnd , IDSIFCONFIG , "Config") ;
    SetItemTranslatedString( hwnd , IDSIFTEST , "Test") ;
    SetItemTranslatedString( hwnd , IDSIFABOUT , "About") ;
    
    SetItemTranslatedString( hwnd , IDAPPLY , "Apply") ;
    SetItemTranslatedString( hwnd , IDOK , "OK") ;
    
    SetWindowTranslatedTitle( hwnd, "Configure Plugins") ; 
    
}

void TranslateGeneralSheet(HWND hwnd) 
{
    SetItemTranslatedString( hwnd , IDC_GENERAL_GSVSYNC , "Wait for V-Sync") ;
}
