/**
 * nSX2 - an OpenSource PS2 Emulator
 * Copyright (C) 2002 Muad
 *
 * homepage: http://nsx2.emulation64.com
 * email address: atreides.muaddib@caramail.com
 * 
 * If you want to contribute to the project please contact
 * me first (maybe someone is already making what you are
 * planning to do).
 *
 *
 * This program is free software; you can redistribute it and/
 * or modify it under the terms of the GNU General Public Li-
 * cence as published by the Free Software Foundation; either
 * version 2 of the Licence, or any later version.
 *
 * This program is distributed in the hope that it will be use-
 * ful, but WITHOUT ANY WARRANTY; without even the implied war-
 * ranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public Licence for more details.
 *
 * You should have received a copy of the GNU General Public
 * Licence along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
**/
#include <stdlib.h>
#include <stdio.h>

#include <math.h>
#include <string.h>


#include "nSX2.h"
#include "PS2Edefs.h"

#include "VU.h"
#include "VU1_micromode_tables.h"

#include "VIF.h"

  

/* Write to VIF1 Registers */
void	VIF1_Write ( _u64 fPhysicalAddress , _u128 fVal , _u8 fDataType ) {

	_u32	boucle;
	_u64	RealAdd;



		RealAdd 		= fPhysicalAddress & 0xFFFF;

		switch ( fDataType ) {

			case DATA_QUAD:	MipsMemoryWriteDW 	( SpecialAreaMemory, fVal.lo, RealAdd );
							MipsMemoryWriteDW 	( SpecialAreaMemory, fVal.hi, RealAdd + 8 );break;

				
			case DATA_DW:	MipsMemoryWriteDW	( SpecialAreaMemory, fVal.lo, RealAdd );break;
				
			case DATA_W:	MipsMemoryWriteW	( SpecialAreaMemory, fVal.lo, RealAdd );break;

			case DATA_HW:	MipsMemoryWriteHW	( SpecialAreaMemory, fVal.lo, RealAdd );break;

			case DATA_B:	MipsMemoryWriteB	( SpecialAreaMemory, fVal.lo, RealAdd );break;
										
			default:		EMU_LOG("%8.8lx:[VIF1][WR][UNDEFINED_TYPE] : 0x%8.8lx ", (long)MipsCpu.PC,fPhysicalAddress);
							MipsCpu.halt = 1;break;
		}





	switch ( fPhysicalAddress  ) {

		case VIF1_STAT_REG: 	  //Status
#ifdef VIF_LOG
			VIF_LOG("%8.8lx:[VIF1][WR]VIF1_STAT : 0x%8.8lx ", (long)MipsCpu.PC,fVal.lo);
#endif
			

		break;

		case VIF1_FBRST_REG: 	  	//Operation Control
#ifdef VIF_LOG
			VIF_LOG("%8.8lx:[VIF1][WR]VIF1_FBRST : 0x%8.8lx ", (long)MipsCpu.PC,fVal.lo);
#endif
			switch ( VIF[ VIF1 ].Reg->fbrst ) {

				case 0x1: 
#ifdef VIF_LOG_L2
					VIF_LOG_L2("%8.8lx:[VIF1][WR]VIF1_FBRST : RESET ", (long)MipsCpu.PC);
#endif
					VIF[ VIF1 ].Reg->fbrst = VIF[ VIF1 ].Reg->fbrst & ~(0x1);
				break;

				default:
#ifdef VIF_LOG_L2
					VIF_LOG_L2("%8.8lx:[VIF1][WR]VIF1_FBRST : UNKNOW ( %2.2x ) ", (long)MipsCpu.PC,fVal.lo);
#endif
				break;
			}

		break;

		case VIF1_ERR_REG: 	  	//Error status
#ifdef VIF_LOG
			VIF_LOG("%8.8lx:[VIF1][WR]VIF1_ERR : 0x%8.8lx ", (long)MipsCpu.PC,fVal.lo);
#endif

			VIF[ VIF1 ].iBitInterruptEnable 		= (~(VIF[ VIF1 ].Reg->err)&0x1);
			VIF[ VIF1 ].IgnoreMismatchDmaTAGError	= VIF[ VIF1 ].Reg->err & 0x2;
			VIF[ VIF1 ].IgnoreVIFCodeError			= VIF[ VIF1 ].Reg->err & 0x4;

#ifdef VIF_LOG_L2
			if ( VIF[ VIF1 ].iBitInterruptEnable ) 
				VIF_LOG_L2("%8.8lx:[VIF1][WR]VIF1_ERR : ibit Enable ", (long)MipsCpu.PC);
			else
				VIF_LOG_L2("%8.8lx:[VIF1][WR]VIF1_ERR : ibit Disable ", (long)MipsCpu.PC);

			if ( VIF[ VIF1 ].IgnoreMismatchDmaTAGError ) 
				VIF_LOG_L2("%8.8lx:[VIF1][WR]VIF1_ERR : Ignore DMATAG ERROR ", (long)MipsCpu.PC);
			else
				VIF_LOG_L2("%8.8lx:[VIF1][WR]VIF1_ERR : stall when  DMATAG ERROR ", (long)MipsCpu.PC);

			if ( VIF[ VIF1 ].IgnoreVIFCodeError ) 
				VIF_LOG_L2("%8.8lx:[VIF1][WR]VIF1_ERR : Ignore VIFCode ERROR (as NOP) ", (long)MipsCpu.PC);
			else
				VIF_LOG_L2("%8.8lx:[VIF1][WR]VIF1_ERR : stall when VIFCode ERROR ", (long)MipsCpu.PC);
#endif
		break;


		case VIF1_MARK_REG: 	  	//Mark Value
#ifdef VIF_LOG
			VIF_LOG("%8.8lx:[VIF1][WR]VIF1_MARK : 0x%8.8lx ", (long)MipsCpu.PC,fVal.lo);
#endif
			//VIF[ VIF1 ].Reg->stat = VIF[ VIF1 ].Reg->stat & ~(0x40);

		break;


#ifdef VIF_LOG
		case VIF1_CYCLE_REG: 	  	//Data write cycle
			VIF_LOG("%8.8lx:[VIF1][WR]VIF1_CYCLE : 0x%8.8lx ", (long)MipsCpu.PC,fVal.lo);
		break;

		case VIF1_MODE_REG: 	  	//ADD mode
			VIF_LOG("%8.8lx:[VIF1][WR]VIF1_MODE : 0x%8.8lx ", (long)MipsCpu.PC,fVal.lo);
		break;

		case VIF1_NUM_REG: 	  	//Amount of non-transferred data
			VIF_LOG("%8.8lx:[VIF1][WR]VIF1_NUM : 0x%8.8lx ", (long)MipsCpu.PC,fVal.lo);
		break;

		case VIF1_MASK_REG: 	  	//Write mask pattern
			VIF_LOG("%8.8lx:[VIF1][WR]VIF1_MASK : 0x%8.8lx ", (long)MipsCpu.PC,fVal.lo);
		break;

		case VIF1_CODE_REG: 	  	//Last processed VIFcode
			VIF_LOG("%8.8lx:[VIF1][WR]VIF1_CODE : 0x%8.8lx ", (long)MipsCpu.PC,fVal.lo);
		break;

		case VIF1_ITOPS_REG: 	  	//Next ITOP value
			VIF_LOG("%8.8lx:[VIF1][WR]VIF1_ITOPS : 0x%8.8lx ", (long)MipsCpu.PC,fVal.lo);
		break;

		case VIF1_BASE_REG: 	  	//Base address of double buffer
			VIF_LOG("%8.8lx:[VIF1][WR]VIF1_BASE : 0x%8.8lx ", (long)MipsCpu.PC,fVal.lo);
		break;
#endif

		case VIF1_OFST_REG: 	  	//Offset of double buffer
#ifdef VIF_LOG
			VIF_LOG("%8.8lx:[VIF1][WR]VIF1_OFST : 0x%8.8lx ", (long)MipsCpu.PC,fVal.lo);
#endif

			VIF[ VIF1 ].Reg->top = (_u32)VIF[  VIF1 ].Reg->base;
		break;

#ifdef VIF_LOG
		case VIF1_TOPS_REG: 	  	//Next TOP value/data write address
			VIF_LOG("%8.8lx:[VIF1][WR]VIF1_TOPS : 0x%8.8lx ", (long)MipsCpu.PC,fVal.lo);
		break;

		case VIF1_ITOP_REG: 	  	//ITOP value
			DMAC_LOG("%8.8lx:[VIF1][WR]VIF1_ITOP : 0x%8.8lx ", (long)MipsCpu.PC,fVal.lo);
		break;

		case VIF1_TOP_REG: 	  	//TOP value
			VIF_LOG("%8.8lx:[VIF1][WR]VIF1_TOP : 0x%8.8lx ", (long)MipsCpu.PC,fVal.lo);
		break;

		case VIF1_R0: 	  	//Filling data R0 (Row register)
			VIF_LOG("%8.8lx:[VIF1][WR]VIF1_R0 : 0x%8.8lx ", (long)MipsCpu.PC,fVal.lo);
		break;

		case VIF1_R1: 	  	//Filling data R1 (Row register)
			VIF_LOG("%8.8lx:[VIF1][WR]VIF1_R1 : 0x%8.8lx ", (long)MipsCpu.PC,fVal.lo);
		break;

		case VIF1_R2: 	  	//Filling data R2 (Row register)
			VIF_LOG("%8.8lx:[VIF1][WR]VIF1_R2 : 0x%8.8lx ", (long)MipsCpu.PC,fVal.lo);
		break;

		case VIF1_R3: 	  	//Filling data R3 (Row register)
			VIF_LOG("%8.8lx:[VIF1][WR]VIF1_R3 : 0x%8.8lx ", (long)MipsCpu.PC,fVal.lo);
		break;

		case VIF1_C0: 	  	//Filling data C0 (Col register)
			VIF_LOG("%8.8lx:[VIF1][WR]VIF1_C0 : 0x%8.8lx ", (long)MipsCpu.PC,fVal.lo);
		break;

		case VIF1_C1: 	  	//Filling data C1 (Col register)
			VIF_LOG("%8.8lx:[VIF1][WR]VIF1_C1 : 0x%8.8lx ", (long)MipsCpu.PC,fVal.lo);
		break;

		case VIF1_C2: 	  	//Filling data C2 (Col register)
			VIF_LOG("%8.8lx:[VIF1][WR]VIF1_C2 : 0x%8.8lx ", (long)MipsCpu.PC,fVal.lo);
		break;

		case VIF1_C3: 	  	//Filling data C3 (Col register)
			VIF_LOG("%8.8lx:[VIF1][WR]VIF1_C3 : 0x%8.8lx ", (long)MipsCpu.PC,fVal.lo);
		break;


		default: 

			EMU_LOG("%8.8lx:[VIF1][WR]???? : 0x%8.8lx ", (long)MipsCpu.PC,fPhysicalAddress);

		break;
#endif
	

	}
}


/* Read from VIF1 Registers */
void	VIF1_Read ( _u64 fPhysicalAddress , _u128 * fVal , _u8 fDataType ) {

	_u32	boucle;
	_u64	RealAdd;


#ifdef VIF_LOG
	switch ( fPhysicalAddress  ) {

		case VIF1_STAT_REG: 	  //Status
			VIF_LOG("%8.8lx:[VIF1][RD]VIF1_STAT : 0x%8.8lx ", (long)MipsCpu.PC,VIF[ VIF1 ].Reg->stat);
		break;

		case VIF1_FBRST_REG: 	  	//Operation Control
			VIF_LOG("%8.8lx:[VIF1][RD]VIF1_FBRST : 0x%8.8lx ", (long)MipsCpu.PC,VIF[ VIF1 ].Reg->fbrst);
		break;

		case VIF1_ERR_REG: 	  	//Error status
			VIF_LOG("%8.8lx:[VIF1][RD]VIF1_ERR : 0x%8.8lx ", (long)MipsCpu.PC,VIF[ VIF1 ].Reg->err);
		break;

		case VIF1_MARK_REG: 	  	//Mark Value
			VIF_LOG("%8.8lx:[VIF1][RD]VIF1_MARK : 0x%8.8lx ", (long)MipsCpu.PC,VIF[ VIF1 ].Reg->mark);
		break;

		case VIF1_CYCLE_REG: 	  	//Data write cycle
			VIF_LOG("%8.8lx:[VIF1][RD]VIF1_CYCLE : 0x%8.8lx ", (long)MipsCpu.PC,VIF[ VIF1 ].Reg->cycle);
		break;

		case VIF1_MODE_REG: 	  	//ADD mode
			VIF_LOG("%8.8lx:[VIF1][RD]VIF1_MODE : 0x%8.8lx ", (long)MipsCpu.PC,VIF[ VIF1 ].Reg->mode);
		break;

		case VIF1_NUM_REG: 	  	//Amount of non-transferred data
			VIF_LOG("%8.8lx:[VIF1][RD]VIF1_NUM : 0x%8.8lx ", (long)MipsCpu.PC,VIF[ VIF1 ].Reg->num);
		break;

		case VIF1_MASK_REG: 	  	//Write mask pattern
			VIF_LOG("%8.8lx:[VIF1][RD]VIF1_MASK : 0x%8.8lx ", (long)MipsCpu.PC,VIF[ VIF1 ].Reg->mask);
		break;

		case VIF1_CODE_REG: 	  	//Last processed VIFcode
			VIF_LOG("%8.8lx:[VIF1][RD]VIF1_CODE : 0x%8.8lx ", (long)MipsCpu.PC,VIF[ VIF1 ].Reg->code);
		break;

		case VIF1_ITOPS_REG: 	  	//Next ITOP value
			VIF_LOG("%8.8lx:[VIF1][RD]VIF1_ITOPS : 0x%8.8lx ", (long)MipsCpu.PC,VIF[ VIF1 ].Reg->itops);
		break;

		case VIF1_BASE_REG: 	  	//Base address of double buffer
			VIF_LOG("%8.8lx:[VIF1][RD]VIF1_BASE : 0x%8.8lx ", (long)MipsCpu.PC,VIF[ VIF1 ].Reg->base);
		break;

		case VIF1_OFST_REG: 	  	//Offset of double buffer
			VIF_LOG("%8.8lx:[VIF1][RD]VIF1_OFST : 0x%8.8lx ", (long)MipsCpu.PC,VIF[ VIF1 ].Reg->ofst);
		break;

		case VIF1_TOPS_REG: 	  	//Next TOP value/data write address
			VIF_LOG("%8.8lx:[VIF1][RD]VIF1_TOPS : 0x%8.8lx ", (long)MipsCpu.PC,VIF[ VIF1 ].Reg->tops);
		break;

		case VIF1_ITOP_REG: 	  	//ITOP value
			DMAC_LOG("%8.8lx:[VIF1][RD]VIF1_ITOP : 0x%8.8lx ", (long)MipsCpu.PC,VIF[ VIF1 ].Reg->itop);
		break;

		case VIF1_TOP_REG: 	  	//TOP value
			VIF_LOG("%8.8lx:[VIF1][RD]VIF1_TOP : 0x%8.8lx ", (long)MipsCpu.PC,VIF[ VIF1 ].Reg->top);
		break;

		case VIF1_R0: 	  	//Filling data R0 (Row register)
			VIF_LOG("%8.8lx:[VIF1][RD]VIF1_R0 : 0x%8.8lx ", (long)MipsCpu.PC,VIF[ VIF1 ].Reg->r0);
		break;

		case VIF1_R1: 	  	//Filling data R1 (Row register)
			VIF_LOG("%8.8lx:[VIF1][RD]VIF1_R1 : 0x%8.8lx ", (long)MipsCpu.PC,VIF[ VIF1 ].Reg->r1);
		break;

		case VIF1_R2: 	  	//Filling data R2 (Row register)
			VIF_LOG("%8.8lx:[VIF1][RD]VIF1_R2 : 0x%8.8lx ", (long)MipsCpu.PC,VIF[ VIF1 ].Reg->r2);
		break;

		case VIF1_R3: 	  	//Filling data R3 (Row register)
			VIF_LOG("%8.8lx:[VIF1][RD]VIF1_R3 : 0x%8.8lx ", (long)MipsCpu.PC,VIF[ VIF1 ].Reg->r3);
		break;

		case VIF1_C0: 	  	//Filling data C0 (Col register)
			VIF_LOG("%8.8lx:[VIF1][RD]VIF1_C0 : 0x%8.8lx ", (long)MipsCpu.PC,VIF[ VIF1 ].Reg->c0);
		break;

		case VIF1_C1: 	  	//Filling data C1 (Col register)
			VIF_LOG("%8.8lx:[VIF1][RD]VIF1_C1 : 0x%8.8lx ", (long)MipsCpu.PC,VIF[ VIF1 ].Reg->c1);
		break;

		case VIF1_C2: 	  	//Filling data C2 (Col register)
			VIF_LOG("%8.8lx:[VIF1][RD]VIF1_C2 : 0x%8.8lx ", (long)MipsCpu.PC,VIF[ VIF1 ].Reg->c2);
		break;

		case VIF1_C3: 	  	//Filling data C3 (Col register)
			VIF_LOG("%8.8lx:[VIF1][RD]VIF1_C3 : 0x%8.8lx ", (long)MipsCpu.PC,VIF[ VIF1 ].Reg->c3);
		break;


		default: {
			EMU_LOG("%8.8lx:[VIF1][RD]???? : 0x%8.8lx ", (long)MipsCpu.PC,fPhysicalAddress);
		break;
		}

	}
#endif

	RealAdd 		= fPhysicalAddress & 0xFFFF;

	switch ( fDataType ) {

		case DATA_QUAD:	MipsMemoryReadDQ 	( SpecialAreaMemory, fVal, RealAdd );break;
					
		case DATA_DW:	MipsMemoryReadDW	( SpecialAreaMemory, fVal, RealAdd );break;

	    case DATA_HW:	MipsMemoryReadHW	( SpecialAreaMemory, fVal, RealAdd );break;
		
		case DATA_W:	MipsMemoryReadW 	( SpecialAreaMemory, fVal, RealAdd );break;

		case DATA_B:	MipsMemoryReadB		( SpecialAreaMemory, fVal, RealAdd );break;

		default:		EMU_LOG("%8.8lx:[VIF1][RD][UNDEFINED_TYPE] : 0x%8.8lx ", (long)MipsCpu.PC,fPhysicalAddress);
						MipsCpu.halt = 1;break;
	}






}

















