/**
 * nSX2 - an OpenSource PS2 Emulator
 * Copyright (C) 2002 Muad
 *
 * homepage: http://nsx2.emulation64.com
 * email address: atreides.muaddib@caramail.com
 * 
 * If you want to contribute to the project please contact
 * me first (maybe someone is already making what you are
 * planning to do).
 *
 *
 * This program is free software; you can redistribute it and/
 * or modify it under the terms of the GNU General Public Li-
 * cence as published by the Free Software Foundation; either
 * version 2 of the Licence, or any later version.
 *
 * This program is distributed in the hope that it will be use-
 * ful, but WITHOUT ANY WARRANTY; without even the implied war-
 * ranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public Licence for more details.
 *
 * You should have received a copy of the GNU General Public
 * Licence along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
**/
#include <stdlib.h>
#include <stdio.h>

#include <math.h>
#include <string.h>


#include "nSX2.h"
#include "PS2Edefs.h"

#include "VU.h"
#include "VU1_micromode_tables.h"

#include "VIF.h"


/********************************************************************************************/
/*																		*/
/********************************************************************************************/

  

/* 
Copy Data from EE Memory to VU Memory 

Params:
CurrentData : is the current VIF Command
fUnpackDataSize : is the previously calculated size with VIF1_ComputePackLenght()

*/


/* UNPACK_V4_32 */
void VIF_Unpack_V4_32( _u8 fVIF,_u32 * CurrentData , _u32 fUnpackDataSize) {

	_u8	IsSigned;		//datas must by Signed
	_u8 FLG;			//Addr is: ADDR + TOPS

	_u32 * dataSrc;		//pointer to data src ( Mips Memory )
	_u32 * dataDest;	//pointer to data dest ( VectorUnit Memory )
	_u32 dataDestAdd;	//Addr of Destination Data in the VectorUnit1 Memory


	/* Extract Field from the VIF1 Command */
	dataDestAdd = ((* CurrentData) & 0x3FF ) * 16;
	IsSigned 	= ((* CurrentData) >> 14 ) & 0x1;
	FLG 		= ((* CurrentData) >> 15 ) & 0x1;

	/* Test if I must use the TOPS Register */
	if (FLG == 1) {
	
		/*	Why I multiply the TOPS value by 16 ?
				At the Activation of the MicroProgram, the TOPS value is to TOP.
				the XTOP instruction get the TOP Register
				If this value is used to get data from VU mem with an instruction
				like ILWR , the value is muliplied by 16
				So if I want the data is at the right place ,
				multiply the TOPS value by 16

				Exemple from the basic_vu1.elf

					00a0:XTOP VI[1] 
					 |VI[1] = 00000200.
					00ac:NOP
					00a8:ILWR.8 VI[10] (VI[1].8)
					 | VI[1] = 0200  Addr = 00002000.

				
		*/	
		dataDestAdd = (_u32)((_u32)(VIF[ fVIF ].Reg->tops)  * 16 ) + dataDestAdd;
	}



	/* Print Debug Info */
#ifdef VIF_LOG_TR
	VIF_LOG_TR("[VIF%1.1d][TR]VU.mem add : 0x%8.8x , size : %d (_u32), signed : %d , FLG : %d (TOPS=0x%4.4x)", 
			fVIF,dataDestAdd,fUnpackDataSize,IsSigned,FLG,VIF[ fVIF ].Reg->tops);
#endif

	/* The first data is just after the command */
	dataSrc = CurrentData + 1;

	/* Compute the destination pointer */	
	switch ( fVIF ) {
		case VIF0: dataDest = (_u32*)(VectorUnit0.Mem + dataDestAdd);break;
		case VIF1: dataDest = (_u32*)(VectorUnit1.Mem + dataDestAdd);break;
	}

	/* Copy datas ( sizeof(_u32) = 4 ) */
	memcpy ( dataDest, dataSrc, sizeof(_u32) *  fUnpackDataSize  );
}




