/**
 * nSX2 - an OpenSource PS2 Emulator
 * Copyright (C) 2002 Muad
 *
 * homepage: http://nsx2.emulation64.com
 * email address: atreides.muaddib@caramail.com
 * 
 * If you want to contribute to the project please contact
 * me first (maybe someone is already making what you are
 * planning to do).
 *
 *
 * This program is free software; you can redistribute it and/
 * or modify it under the terms of the GNU General Public Li-
 * cence as published by the Free Software Foundation; either
 * version 2 of the Licence, or any later version.
 *
 * This program is distributed in the hope that it will be use-
 * ful, but WITHOUT ANY WARRANTY; without even the implied war-
 * ranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public Licence for more details.
 *
 * You should have received a copy of the GNU General Public
 * Licence along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
**/

#include <stdio.h>
#include <stdlib.h>

#include <strings.h>
#include <float.h>
#include <math.h>

#include "nSX2.h"
#include "PS2Edefs.h"
 
#include "r5900_main.h"

#include "VU_opcodes.h"

#include "VU1_micromode_tables.h"


//#include "../PS2Edefs.h"

#include "VU.h"

#include "VU1_micromode_debug.h"


void PrintDebugVUopcodeUpper(void) {

	char 		tempText[100];	
	sprintf(tempText,"VU1_PC:%4.4x:%8.8lx:", VectorUnit1.PC - 4 ,UpperOp );
	switch ( MipsCpu.vuOpcode&0x3F ) {

		case 60 : sprintf(tempText,"%sVU1:UP_60: %s \n", tempText,debug_COP2_MI_VU1_UPPER_T60[(MipsCpu.vuOpcode & 0x7C0)>>6]);break;
		case 61 : sprintf(tempText,"%sVU1:UP_61: %s \n", tempText,debug_COP2_MI_VU1_UPPER_T61[(MipsCpu.vuOpcode & 0x7C0)>>6]);break;
		case 62 : sprintf(tempText,"%sVU1:UP_62: %s \n", tempText,debug_COP2_MI_VU1_UPPER_T62[(MipsCpu.vuOpcode & 0x7C0)>>6]);break;
		case 63: sprintf(tempText,"%sVU1:UP_63: %s \n", tempText,debug_COP2_MI_VU1_UPPER_T63[(MipsCpu.vuOpcode & 0x7C0)>>6]);break;
		default : sprintf(tempText,"%sVU1:UP: %s \n", tempText,debug_COP2_MI_VU1_UPPER[MipsCpu.vuOpcode&0x3F]);break;
	}

	//PrintToEmuOutput(tempText);

}

void PrintDebugVUopcodeLower(void) {

	char 		tempText[100];	
	sprintf(tempText,"VU1_PC:%4.4x:%8.8lx:", VectorUnit1.PC  - 8,LowerOp);

	switch ( ( MipsCpu.vuOpcode >> 25) & 0x7F ) {

		case 64: {
			
			switch ( MipsCpu.vuOpcode&0x3F ) {

				case 60 : sprintf(tempText,"%sVU1:LW_64_60: %s \n", tempText,debug_COP2_MI_VU1_LOWER_T64_60[(MipsCpu.vuOpcode & 0x7C0)>>6]);break;
				case 61 : sprintf(tempText,"%sVU1:LW_64_61: %s \n", tempText,debug_COP2_MI_VU1_LOWER_T64_61[(MipsCpu.vuOpcode & 0x7C0)>>6]);break;
				case 62 : sprintf(tempText,"%sVU1:LW_64_62: %s \n", tempText,debug_COP2_MI_VU1_LOWER_T64_62[(MipsCpu.vuOpcode & 0x7C0)>>6]);break;
				case 63: sprintf(tempText,"%sVU1:LW_64_63: %s \n", tempText,debug_COP2_MI_VU1_LOWER_T64_63[(MipsCpu.vuOpcode & 0x7C0)>>6]);break;
				default : sprintf(tempText,"%sVU1:LW_64: %s \n", tempText,debug_COP2_MI_VU1_LOWER_T64[(MipsCpu.vuOpcode & 0x3F)]);;break;
			}
			break;
		}
		default : sprintf(tempText,"%sVU1:LW: %s \n", tempText,debug_COP2_MI_VU1_LOWER[( MipsCpu.vuOpcode >> 25) & 0x7F]);break;
		}

	//PrintToEmuOutput(tempText);

}


void PrintDebugVUopcode(void) {

	PrintDebugVUopcodeUpper();
	PrintDebugVUopcodeLower();
}


/* LOG Function for the Interrupt Controller*/
void VU_MPG_Log ( char * ftext  , ...   ) {
	char VU_LogText[ FILE_NAME_LENGHT ];


    va_list ap;
	va_start (ap, ftext);
  	vsprintf (debugTempText, ftext, ap);
   	va_end (ap);

	sprintf(debugTempTextVU1,"%s\n",debugTempText);
		



}


void dVU1_NI	(void)	{PrintDebugVUopcode();VectorUnit1.halt = 1;}


void dVU1_COP2_MI_VU1_UPPER_T60	(void){	dCOP2_MI_VU1_UPPER_T60[(MipsCpu.vuOpcode & 0x7C0)>>6 ]();	}
void dVU1_COP2_MI_VU1_UPPER_T61	(void){	dCOP2_MI_VU1_UPPER_T61[(MipsCpu.vuOpcode & 0x7C0)>>6 ]();	}
void dVU1_COP2_MI_VU1_UPPER_T62	(void){	dCOP2_MI_VU1_UPPER_T62[(MipsCpu.vuOpcode & 0x7C0)>>6 ]();	}
void dVU1_COP2_MI_VU1_UPPER_T63	(void){	dCOP2_MI_VU1_UPPER_T63[(MipsCpu.vuOpcode & 0x7C0)>>6 ]();	}


void dVU1_COP2_MI_VU1_LOWER_T64_60(void){dCOP2_MI_VU1_LOWER_T64_60[(MipsCpu.vuOpcode & 0x7C0)>>6]();}
void dVU1_COP2_MI_VU1_LOWER_T64_61(void){dCOP2_MI_VU1_LOWER_T64_61[(MipsCpu.vuOpcode & 0x7C0)>>6]();}
void dVU1_COP2_MI_VU1_LOWER_T64_62(void){dCOP2_MI_VU1_LOWER_T64_62[(MipsCpu.vuOpcode & 0x7C0)>>6]();}
void dVU1_COP2_MI_VU1_LOWER_T64_63(void){dCOP2_MI_VU1_LOWER_T64_63[(MipsCpu.vuOpcode & 0x7C0)>>6]();}
void dVU1_COP2_MI_VU1_LOWER_T64	 (void){dCOP2_MI_VU1_LOWER_T64[(MipsCpu.vuOpcode & 0x3F)]();}



/********************************************************************************/
/*						UPPER OPCODES											*/
/********************************************************************************/

void dVU1_ABS	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: ABS.%s %s.%s, %s.%s ", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], d_vf_reg[CURRENT_FT], d_dest_field[ CURRENT_VU_DEST ],
										 d_vf_reg[CURRENT_FS], d_dest_field[ CURRENT_VU_DEST ]
		);
#endif
}


void dVU1_ADD	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: ADD.%s  %s.%s, %s.%s, %s.%s", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], d_vf_reg[CURRENT_FD], d_dest_field[ CURRENT_VU_DEST ],
										 d_vf_reg[CURRENT_FS], d_dest_field[ CURRENT_VU_DEST ],
										 d_vf_reg[CURRENT_FT], d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_ADDi	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: ADDi.%s  %s.%s, %s.%s, I", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], d_vf_reg[CURRENT_FD], d_dest_field[ CURRENT_VU_DEST ],
										 d_vf_reg[CURRENT_FS], d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_ADDq	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: ADDq.%s  %s.%s, %s.%s, Q", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], d_vf_reg[CURRENT_FD], d_dest_field[ CURRENT_VU_DEST ],
										 d_vf_reg[CURRENT_FS], d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_ADDbc	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: ADD%s.%s  %s.%s, %s.%s, %s.%s", (long)VectorUnit1.PC-4,
		d_bc_field[ CURRENT_VU_BC ],	d_dest_field[ CURRENT_VU_DEST ], 
		d_vf_reg[CURRENT_FD], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FT], 			d_bc_field[ CURRENT_VU_BC ]
		); 
#endif
}


void dVU1_ADDA	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: ADDA.%s  ACC.%s, %s.%s, %s.%s", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], 
								 	d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 		d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FT], 		d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}
 

void dVU1_ADDAq	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: ADDAq.%s  ACC.%s, %s.%s, Q", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], 
								d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 	d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_ADDAi	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: ADDAi.%s  ACC.%s, %s.%s, I", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], 
								d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 	d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_ADDAbc	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: ADDA%s.%s  ACC.%s, %s.%s, %s.%s", (long)VectorUnit1.PC-4,
		d_bc_field[ CURRENT_VU_BC ],	d_dest_field[ CURRENT_VU_DEST ], 
							 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FT], 			d_bc_field[ CURRENT_VU_BC ]
		); 
#endif
}


void dVU1_CLIP	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: CLIPw.%s   %s.%s, %s.w", (long)VectorUnit1.PC-4,
										d_dest_field[ CURRENT_VU_DEST ], 
		d_vf_reg[CURRENT_FS], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FT]
	); 
#endif
}


void dVU1_FTOI0	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: FTOI0.%s  %s.%s, %s.%s,",	(long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FT], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 			d_dest_field[ CURRENT_VU_DEST ]
	);
#endif
}


void dVU1_FTOI4	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: FTOI4.%s  %s.%s, %s.%s,",	(long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FT], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 			d_dest_field[ CURRENT_VU_DEST ]
	);
#endif
}	


void dVU1_FTOI12	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: FTOI12.%s  %s.%s, %s.%s,",	(long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FT], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 			d_dest_field[ CURRENT_VU_DEST ]
	);
#endif

}


void dVU1_FTOI15	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: FTOI15.%s  %s.%s, %s.%s,",	(long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FT], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 			d_dest_field[ CURRENT_VU_DEST ]
	);
#endif
}


void dVU1_ITOF0	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: ITOF0.%s  %s.%s, %s.%s,",	(long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FT], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 			d_dest_field[ CURRENT_VU_DEST ]
	);
#endif
}


void dVU1_ITOF4	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: ITOF4.%s  %s.%s, %s.%s,",	(long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FT], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 			d_dest_field[ CURRENT_VU_DEST ]
	);
#endif
}


void dVU1_ITOF12	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: ITOF12.%s  %s.%s, %s.%s,",	(long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FT], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 			d_dest_field[ CURRENT_VU_DEST ]
	);
#endif
}


void dVU1_ITOF15	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: ITOF15.%s  %s.%s, %s.%s,",	(long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FT], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 			d_dest_field[ CURRENT_VU_DEST ]
	);
#endif
}	


void dVU1_MADD	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MADD.%s  %s.%s, %s.%s, %s.%s", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], d_vf_reg[CURRENT_FD], d_dest_field[ CURRENT_VU_DEST ],
										 d_vf_reg[CURRENT_FS], d_dest_field[ CURRENT_VU_DEST ],
										 d_vf_reg[CURRENT_FT], d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_MADDi	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MADDi.%s  %s.%s, %s.%s, I", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], d_vf_reg[CURRENT_FD], d_dest_field[ CURRENT_VU_DEST ],
										 d_vf_reg[CURRENT_FS], d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_MADDq(void){
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MADDq.%s  %s.%s, %s.%s, Q", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], d_vf_reg[CURRENT_FD], d_dest_field[ CURRENT_VU_DEST ],
										 d_vf_reg[CURRENT_FS], d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_MADDbc	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MADDA%s.%s  %s.%s, %s.%s, %s.%s", (long)VectorUnit1.PC-4,
		d_bc_field[ CURRENT_VU_BC ],	d_dest_field[ CURRENT_VU_DEST ], 
		d_vf_reg[CURRENT_FD], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FT], 			d_bc_field[ CURRENT_VU_BC ]
		); 
#endif
}


void dVU1_MADDA	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%8.8llx: MADDA.%s  ACC.%s, %s.%s, %s.%s", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], 
								 	d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 		d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FT], 		d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_MADDAi	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MADDAi.%s  ACC.%s, %s.%s, I", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], 
								d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 	d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_MADDAq	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MADDAq.%s  ACC.%s, %s.%s, Q", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], 
								d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 	d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_MADDAbc(void)	{ 	
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MADDA%s.%s  ACC.%s, %s.%s, %s.%s", (long)VectorUnit1.PC-4,
		d_bc_field[ CURRENT_VU_BC ],	d_dest_field[ CURRENT_VU_DEST ], 
							 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FT], 			d_bc_field[ CURRENT_VU_BC ]
		); 
#endif
}


void dVU1_MAXi(void){
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MAXi.%s  %s.%s, %s.%s, I", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], d_vf_reg[CURRENT_FD], d_dest_field[ CURRENT_VU_DEST ],
										 d_vf_reg[CURRENT_FS], d_dest_field[ CURRENT_VU_DEST ]
										 
		); 
#endif
}


void dVU1_MAX	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: VMAX.%s  %s.%s, %s.%s, %s.%s", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], d_vf_reg[CURRENT_FD], d_dest_field[ CURRENT_VU_DEST ],
										 d_vf_reg[CURRENT_FS], d_dest_field[ CURRENT_VU_DEST ],
										 d_vf_reg[CURRENT_FT], d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_MAXbc	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MAX%s.%s  %s.%s, %s.%s, %s.%s", (long)VectorUnit1.PC-4,
		d_bc_field[ CURRENT_VU_BC ],	d_dest_field[ CURRENT_VU_DEST ], 
		d_vf_reg[CURRENT_FD], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FT], 			d_bc_field[ CURRENT_VU_BC ]
		); 
#endif
}


void dVU1_MINI	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MINI.%s  %s.%s, %s.%s, %s.%s", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], d_vf_reg[CURRENT_FD], d_dest_field[ CURRENT_VU_DEST ],
										 d_vf_reg[CURRENT_FS], d_dest_field[ CURRENT_VU_DEST ],
										 d_vf_reg[CURRENT_FT], d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_MINIi	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MINIi.%s %s.%s, %s.%s, I", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], d_vf_reg[CURRENT_FD], d_dest_field[ CURRENT_VU_DEST ],
										 d_vf_reg[CURRENT_FS], d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}

	
void dVU1_MINIbc	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MINI%s.%s  %s.%s, %s.%s, %s.%s", (long)VectorUnit1.PC-4,
		d_bc_field[ CURRENT_VU_BC ],	d_dest_field[ CURRENT_VU_DEST ], 
		d_vf_reg[CURRENT_FD], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FT], 			d_bc_field[ CURRENT_VU_BC ]
		); 
#endif
}


void dVU1_MSUB	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MSUB.%s  %s.%s, %s.%s, %s.%s",  (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], d_vf_reg[CURRENT_FD], d_dest_field[ CURRENT_VU_DEST ],
										 d_vf_reg[CURRENT_FS], d_dest_field[ CURRENT_VU_DEST ],
										 d_vf_reg[CURRENT_FT], d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_MSUBi	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MSUBi.%s  %s.%s, %s.%s, I", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], d_vf_reg[CURRENT_FD], d_dest_field[ CURRENT_VU_DEST ],
										 d_vf_reg[CURRENT_FS], d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_MSUBq	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MSUBq.%s  %s.%s, %s.%s, Q", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], d_vf_reg[CURRENT_FD], d_dest_field[ CURRENT_VU_DEST ],
										 d_vf_reg[CURRENT_FS], d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_MSUBbc	(void){ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MSUB%s.%s  %s.%s, %s.%s, %s.%s", (long)VectorUnit1.PC-4,
		d_bc_field[ CURRENT_VU_BC ],	d_dest_field[ CURRENT_VU_DEST ], 
		d_vf_reg[CURRENT_FD], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FT], 			d_bc_field[ CURRENT_VU_BC ]
		); 
#endif
}


void dVU1_MSUBA	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MSUBA.%s  ACC.%s, %s.%s, %s.%s", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], 
								 	d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 		d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FT], 		d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_MSUBAi	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MSUBAi.%s  ACC.%s, %s.%s, I", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], 
								d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 	d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_MSUBAq	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MSUBAq.%s  ACC.%s, %s.%s, Q", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], 
								d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 	d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_MSUBAbc(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MSUBA%s.%s  ACC.%s, %s.%s, %s.%s", (long)VectorUnit1.PC-4,
		d_bc_field[ CURRENT_VU_BC ],	d_dest_field[ CURRENT_VU_DEST ], 
							 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FT], 			d_bc_field[ CURRENT_VU_BC ]
		); 
#endif
}


void dVU1_MUL	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MUL.%s  %s.%s, %s.%s, %s.%s", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], d_vf_reg[CURRENT_FD], d_dest_field[ CURRENT_VU_DEST ],
										 d_vf_reg[CURRENT_FS], d_dest_field[ CURRENT_VU_DEST ],
										 d_vf_reg[CURRENT_FT], d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_MULi	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MULi.%s  %s.%s, %s.%s, I",  (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], d_vf_reg[CURRENT_FD], d_dest_field[ CURRENT_VU_DEST ],
										 d_vf_reg[CURRENT_FS], d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_MULq	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MULq.%s  %s.%s, %s.%s, Q", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], d_vf_reg[CURRENT_FD], d_dest_field[ CURRENT_VU_DEST ],
										 d_vf_reg[CURRENT_FS], d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_MULbc	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MUL%s.%s  %s.%s, %s.%s, %s.%s", (long)VectorUnit1.PC-4,
		d_bc_field[ CURRENT_VU_BC ],	d_dest_field[ CURRENT_VU_DEST ], 
		d_vf_reg[CURRENT_FD], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FT], 			d_bc_field[ CURRENT_VU_BC ]
		); 
#endif
}

void dVU1_MULA	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MULA.%s  ACC.%s, %s.%s, %s.%s", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], 
								 	d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 		d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FT], 		d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_MULAi	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MULAi.%s  ACC.%s, %s.%s, I", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], 
								d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 	d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}	


void dVU1_MULAq	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MULAq.%s  ACC.%s, %s.%s, Q", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], 
								d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 	d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}	


void dVU1_MULAbc	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MULA%s.%s  ACC.%s, %s.%s, %s.%s", (long)VectorUnit1.PC-4,
		d_bc_field[ CURRENT_VU_BC ],	d_dest_field[ CURRENT_VU_DEST ], 
							 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FT], 			d_bc_field[ CURRENT_VU_BC ]
		); 
#endif
}


void dVU1_NOP	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG( "%4.4llx: NOP", (long)VectorUnit1.PC-4  );
#endif
}


void dVU1_OPMULA	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: OPMULA.%s  ACC.%s, %s.%s, %s.%s", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], 
							 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FT], 			d_bc_field[ CURRENT_VU_BC ]
		); 
#endif
}	


void dVU1_OPMSUB	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: OPMSUB.%s  %s.%s, %s.%s, %s.%s", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], 
		d_vf_reg[CURRENT_FD], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FT], 			d_bc_field[ CURRENT_VU_BC ]
		); 
#endif
}


void dVU1_SUB	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: SUB.%s  %s.%s, %s.%s, %s.%s", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], d_vf_reg[CURRENT_FD], d_dest_field[ CURRENT_VU_DEST ],
										 d_vf_reg[CURRENT_FS], d_dest_field[ CURRENT_VU_DEST ],
										 d_vf_reg[CURRENT_FT], d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_SUBi	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: SUBi.%s  %s.%s, %s.%s, I", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], d_vf_reg[CURRENT_FD], d_dest_field[ CURRENT_VU_DEST ],
										 d_vf_reg[CURRENT_FS], d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}	


void dVU1_SUBq	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: SUBq.%s  %s.%s, %s.%s, Q",  (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], d_vf_reg[CURRENT_FD], d_dest_field[ CURRENT_VU_DEST ],
										 d_vf_reg[CURRENT_FS], d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_SUBbc	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: SUB%s.%s  %s.%s, %s.%s, %s.%s", (long)VectorUnit1.PC-4,
		d_bc_field[ CURRENT_VU_BC ],	d_dest_field[ CURRENT_VU_DEST ], 
		d_vf_reg[CURRENT_FD], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FT], 			d_bc_field[ CURRENT_VU_BC ]
		); 
#endif
}


void dVU1_SUBBA	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: SUBA.%s  ACC.%s, %s.%s, %s.%s", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], 
								 	d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 		d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FT], 		d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}

	
void dVU1_SUBAi	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: SUBAi.%s  ACC.%s, %s.%s, I", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], 
								d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 	d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_SUBAq	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: SUBAq.%s  ACC.%s, %s.%s, Q", (long)VectorUnit1.PC-4,
		d_dest_field[ CURRENT_VU_DEST ], 
								d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 	d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_SUBAbc	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: SUBA%s.%s  ACC.%s, %s.%s, %s.%s", (long)VectorUnit1.PC-4,
		d_bc_field[ CURRENT_VU_BC ],	d_dest_field[ CURRENT_VU_DEST ], 
							 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FS], 			d_dest_field[ CURRENT_VU_DEST ],
		d_vf_reg[CURRENT_FT], 			d_bc_field[ CURRENT_VU_BC ]
		); 
#endif
}


/********************************************************************************/
/*						LOWER OPCODES											*/
/********************************************************************************/




void dVU1_B		(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: B  0x%6.6x ", (long)VectorUnit1.PC-8,
			(_VU_OPCODE_ & 0x07FF) 	);		
#endif
}


void dVU1_BAL	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: BAL %s, 0x%6.6x ", (long)VectorUnit1.PC-8,
			debug_vu_ccr_reg[ CURRENT_VU_IT ], (_VU_OPCODE_ & 0x07FF) 	);		
#endif
}
 

void dVU1_DIV	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: DIV  Q  %s.%s, %s.%s ", (long)VectorUnit1.PC-8,
			d_vf_reg[CURRENT_FS], 			d_fstf_field[ CURRENT_VU_FSF ],
			d_vf_reg[CURRENT_FT], 			d_fstf_field[ CURRENT_VU_FTF ]
	);		
#endif
}


void dVU1_EATAN	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: EATAN  P,  %s.%s ", (long)VectorUnit1.PC-8,
			d_vf_reg[CURRENT_FS], 			d_fstf_field[ CURRENT_VU_FSF ]
			
	);		
#endif
}


void dVU1_EATANxy(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: EATANxy  P,  %s ", (long)VectorUnit1.PC-8,
			d_vf_reg[CURRENT_FS]
			
	);		
#endif
}


void dVU1_EATANxz(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: EATANxz  P,  %s ", (long)VectorUnit1.PC-8,
			d_vf_reg[CURRENT_FS]
			
	);		
#endif
}


void dVU1_EEXP	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: EEXP  P,  %s.%s ", (long)VectorUnit1.PC-8,
			d_vf_reg[CURRENT_FS], 			d_fstf_field[ CURRENT_VU_FSF ]
			
	);		
#endif
}


void dVU1_ELENG	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: ELENG  P,  %s ", (long)VectorUnit1.PC-8,
			d_vf_reg[CURRENT_FS]
			
	);		
#endif
}	


void dVU1_ERCPR	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: ERCPR  P,  %s.%s ", (long)VectorUnit1.PC-8,
			d_vf_reg[CURRENT_FS], 			d_fstf_field[ CURRENT_VU_FSF ]
			
	);		
#endif
}


void dVU1_ERLENG(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: ERLENG  P,  %s ", (long)VectorUnit1.PC-8,
			d_vf_reg[CURRENT_FS]
			
	);		
#endif
}


void dVU1_ERSADD(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: ERSADD  P,  %s ", (long)VectorUnit1.PC-8,
			d_vf_reg[CURRENT_FS]
			
	);		
#endif
}


void dVU1_ERSQRT(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: ERSQRT  P,  %s.%s ", (long)VectorUnit1.PC-8,
			d_vf_reg[CURRENT_FS], 			d_fstf_field[ CURRENT_VU_FSF ]
			
	);		
#endif
}


void dVU1_ESADD	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: ESADD  P,  %s ", (long)VectorUnit1.PC-8,
			d_vf_reg[CURRENT_FS]
			
	);		
#endif
}


void dVU1_ESIN	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: ESIN  P,  %s.%s ", (long)VectorUnit1.PC-8,
			d_vf_reg[CURRENT_FS], 			d_fstf_field[ CURRENT_VU_FSF ]
			
	);		
#endif
}


void dVU1_ESQRT	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: ESQRT  P,  %s.%s ", (long)VectorUnit1.PC-8,
			d_vf_reg[CURRENT_FS], 			d_fstf_field[ CURRENT_VU_FSF ]
			
	);		
#endif
}


void dVU1_ESUM	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: ESUM  P,  %s ", (long)VectorUnit1.PC-8,
			d_vf_reg[CURRENT_FS]
			
	);		
#endif
}


void dVU1_FCAND	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: FCAND %s, 0x%6.6x ", (long)VectorUnit1.PC-8,
			debug_vu_ccr_reg[ 1 ], (_VU_OPCODE_ & 0xFFFFFF) 	
	);		
#endif
}


void dVU1_FCEQ	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: FCEQ %s, 0x%6.6x ", (long)VectorUnit1.PC-8,
			debug_vu_ccr_reg[ 1 ], (_VU_OPCODE_ & 0xFFFFFF) 	
	);		
#endif
}


void dVU1_FCGET	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: FCGET %s ", (long)VectorUnit1.PC-8,
			debug_vu_ccr_reg[ CURRENT_VU_IT ]	
	);		
#endif
}


void dVU1_FCOR	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: FCOR %s, 0x%6.6x ", (long)VectorUnit1.PC-8,
			debug_vu_ccr_reg[ 1 ], (_VU_OPCODE_ & 0xFFFFFF) 	
	);		
#endif
}


void dVU1_FCSET	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: FCSET  0x%6.6x ", (long)VectorUnit1.PC-8,
			(_VU_OPCODE_ & 0xFFFFFF) 	
	);		
#endif
}


void dVU1_FMAND	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: FMAND %s,%s ", (long)VectorUnit1.PC-8,
			debug_vu_ccr_reg[ CURRENT_VU_IT ],	debug_vu_ccr_reg[ CURRENT_VU_IS ]
	);		
#endif
}


void dVU1_FMEQ	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: FMEQ %s,%s ", (long)VectorUnit1.PC-8,
			debug_vu_ccr_reg[ CURRENT_VU_IT ],	debug_vu_ccr_reg[ CURRENT_VU_IS ]
	);		
#endif
}


void dVU1_FMOR	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: FMOR %s,%s ", (long)VectorUnit1.PC-8,
			debug_vu_ccr_reg[ CURRENT_VU_IT ],	debug_vu_ccr_reg[ CURRENT_VU_IS ]
	);		
#endif
}


void dVU1_FSAND	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: FSAND %s, 0x%4.4x ", (long)VectorUnit1.PC-8,
			debug_vu_ccr_reg[ CURRENT_VU_IT ], (_VU_OPCODE_ & 0xFFF) 	
	);		
#endif
}


void dVU1_FSEQ	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: FSEQ %s, 0x%4.4x ", (long)VectorUnit1.PC-8,
			debug_vu_ccr_reg[ CURRENT_VU_IT ], (_VU_OPCODE_ & 0xFFF) 	
	);		
#endif
}


void dVU1_FSOR	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: FSOR %s, 0x%4.4x ", (long)VectorUnit1.PC-8,
			debug_vu_ccr_reg[ CURRENT_VU_IT ], (_VU_OPCODE_ & 0xFFF) 	
	);		
#endif
}


void dVU1_FSSET	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: FSSET  0x%4.4x ", (long)VectorUnit1.PC-8,
			(_VU_OPCODE_ & 0xFFF) 	
	);		
#endif
}


void dVU1_IADD	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: IADD  %s, %s, %s",	(long)VectorUnit1.PC-8,
		debug_vu_ccr_reg[CURRENT_VU_ID],
		debug_vu_ccr_reg[CURRENT_VU_IS], 		
		debug_vu_ccr_reg[CURRENT_VU_IT] 		
	);
#endif
}


void dVU1_IADDI	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: IADDI  %s, %s, 0x%4.4x",	(long)VectorUnit1.PC-8,
		debug_vu_ccr_reg[CURRENT_VU_IT],
		debug_vu_ccr_reg[CURRENT_VU_IS], 		
		((_VU_OPCODE_ >> 6) & 0x1f)		
	);
#endif
}


void dVU1_IADDIU	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: IADDIU  %s, %s, 0x%6.6x",	(long)VectorUnit1.PC-8,
		debug_vu_ccr_reg[CURRENT_VU_IT],
		debug_vu_ccr_reg[CURRENT_VU_IS], 		
		(((_VU_OPCODE_ >> 10) & 0x7800) | (_VU_OPCODE_ & 0x7ff))		
	);
#endif
}



void dVU1_IAND	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: IAND  %s, %s, %s",(long)VectorUnit1.PC-8,
		debug_vu_ccr_reg[CURRENT_VU_ID],
		debug_vu_ccr_reg[CURRENT_VU_IS], 		
		debug_vu_ccr_reg[CURRENT_VU_IT] 		
	);
#endif
}


void dVU1_IBEQ	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: IBEQ  %s, %s, 0x%4.4x",	(long)VectorUnit1.PC-8,
		debug_vu_ccr_reg[CURRENT_VU_IT],
		debug_vu_ccr_reg[CURRENT_VU_IS], 		
		(_VU_OPCODE_ & 0x7FF)		
	);
#endif
}


void dVU1_IBGEZ	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: IBGEZ  %s, 0x%4.4x",	(long)VectorUnit1.PC-8,
		debug_vu_ccr_reg[CURRENT_VU_IS], 		
		(_VU_OPCODE_ & 0x7FF)		
	);
#endif
}


void dVU1_IBGTZ	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: IBGTZ  %s, 0x%4.4x",	(long)VectorUnit1.PC-8,
		debug_vu_ccr_reg[CURRENT_VU_IS], 		
		(_VU_OPCODE_ & 0x7FF)		
	);
#endif
}


void dVU1_IBLEZ	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: IBLEZ  %s, 0x%4.4x",	(long)VectorUnit1.PC-8,
		debug_vu_ccr_reg[CURRENT_VU_IS], 		
		(_VU_OPCODE_ & 0x7FF)		
	);
#endif
}


void dVU1_IBLTZ	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: IBLTZ  %s, 0x%4.4x",	(long)VectorUnit1.PC-8,
		debug_vu_ccr_reg[CURRENT_VU_IS], 		
		(_VU_OPCODE_ & 0x7FF)		
	);
#endif
}


void dVU1_IBNE	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: IBNE  %s, %s, 0x%4.4x",	(long)VectorUnit1.PC-8,
		debug_vu_ccr_reg[CURRENT_VU_IT],
		debug_vu_ccr_reg[CURRENT_VU_IS], 		
		(_VU_OPCODE_ & 0x7FF)		
	);
#endif
}


void dVU1_ILW	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: ILW.%s  %s, 0x%4.4x(%s)%s",	(long)VectorUnit1.PC-8,
		d_dest_field[ CURRENT_VU_DEST ],
		debug_vu_ccr_reg[CURRENT_VU_IT],
		(_VU_OPCODE_ & 0x7FF),
		debug_vu_ccr_reg[CURRENT_VU_IS], 		
		d_dest_field[ CURRENT_VU_DEST ] 		
	);
#endif
}


void dVU1_ILWR	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: ILWR.%s  %s, (%s)%s",	(long)VectorUnit1.PC-8,
		d_dest_field[ CURRENT_VU_DEST ],
		debug_vu_ccr_reg[CURRENT_VU_IT],
		debug_vu_ccr_reg[CURRENT_VU_IS], 		
		d_dest_field[ CURRENT_VU_DEST ] 		
	);
#endif
}


void dVU1_IOR	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: IOR  %s, %s, %s",	(long)VectorUnit1.PC-8,
		debug_vu_ccr_reg[CURRENT_VU_ID],
		debug_vu_ccr_reg[CURRENT_VU_IS], 		
		debug_vu_ccr_reg[CURRENT_VU_IT] 		
	);
#endif
}


void dVU1_ISW	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: ISW.%s  %s, 0x%4.4x(%s)%s",	(long)VectorUnit1.PC-8,
		d_dest_field[ CURRENT_VU_DEST ],
		debug_vu_ccr_reg[CURRENT_VU_IT],
		(_VU_OPCODE_ & 0x7FF),
		debug_vu_ccr_reg[CURRENT_VU_IS], 		
		d_dest_field[ CURRENT_VU_DEST ] 		
	);
#endif
}


void dVU1_ISUB	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: ISUB  %s, %s, %s",	(long)VectorUnit1.PC-8,
		debug_vu_ccr_reg[CURRENT_VU_ID],
		debug_vu_ccr_reg[CURRENT_VU_IS], 		
		debug_vu_ccr_reg[CURRENT_VU_IT] 		
	);
#endif
}


void dVU1_ISUBIU	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: ISUBIU  %s, %s, 0x%6.6x",	(long)VectorUnit1.PC-8,
		debug_vu_ccr_reg[CURRENT_VU_IT],
		debug_vu_ccr_reg[CURRENT_VU_IS], 		
		(((_VU_OPCODE_ >> 10) & 0x7800) | (_VU_OPCODE_ & 0x7ff))		
	);
#endif
}


void dVU1_ISWR	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: ISWR.%s  %s, (%s)%s",	(long)VectorUnit1.PC-8,
		d_dest_field[ CURRENT_VU_DEST ],
		debug_vu_ccr_reg[CURRENT_VU_IT],
		debug_vu_ccr_reg[CURRENT_VU_IS], 		
		d_dest_field[ CURRENT_VU_DEST ] 		
	);
#endif
}


void dVU1_JALR	(void)   {
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: JALR  %s, %s",	(long)VectorUnit1.PC-8,
		debug_vu_ccr_reg[CURRENT_VU_IT], 		
		debug_vu_ccr_reg[CURRENT_VU_IS] 		
	);
#endif
}


void dVU1_JR		(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: JR  %s",	(long)VectorUnit1.PC-8,
		debug_vu_ccr_reg[CURRENT_VU_IS] 		
	);
#endif
}


void dVU1_LQ		(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: LQ.%s  %s.%s, 0x%4.4x(%s) ", (long)VectorUnit1.PC-8,
		d_dest_field[ CURRENT_VU_DEST ], 

		d_vf_reg[CURRENT_FT], 			d_dest_field[ CURRENT_VU_DEST ],
		(_VU_OPCODE_ & 0x7FF),
		debug_vu_ccr_reg[CURRENT_VU_IS]
		); 
#endif
}


void dVU1_LQD	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: LQD.%s  %s.%s, (--%s).%s ", (long)VectorUnit1.PC-8,
		d_dest_field[ CURRENT_VU_DEST ], 

		d_vf_reg[CURRENT_FT], 			d_dest_field[ CURRENT_VU_DEST ],
		debug_vu_ccr_reg[CURRENT_VU_IS],d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_LQI	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: LQI.%s  %s.%s, (%s++).%s ", (long)VectorUnit1.PC-8,
		d_dest_field[ CURRENT_VU_DEST ], 

		d_vf_reg[CURRENT_FT], 			d_dest_field[ CURRENT_VU_DEST ],
		debug_vu_ccr_reg[CURRENT_VU_IS],d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_MFIR	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MFIR.%s  %s.%s, %s", (long)VectorUnit1.PC-8,
		d_dest_field[ CURRENT_VU_DEST ], 
		d_vf_reg[CURRENT_FT], d_dest_field[ CURRENT_VU_DEST ],
		debug_vu_ccr_reg[CURRENT_VU_IS]
		); 
#endif
}


void dVU1_MFP	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MFP.%s  %s.%s, P", (long)VectorUnit1.PC-8,
		d_dest_field[ CURRENT_VU_DEST ], 
		d_vf_reg[CURRENT_FT], d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}
 

void dVU1_MOVE	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MOVE.%s  %s.%s, %s.%s", (long)VectorUnit1.PC-8,
		d_dest_field[ CURRENT_VU_DEST ], d_vf_reg[CURRENT_FT], d_dest_field[ CURRENT_VU_DEST ],
										 d_vf_reg[CURRENT_FS], d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_MR32	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: MR32.%s  %s.%s, %s.%s", (long)VectorUnit1.PC-8,
		d_dest_field[ CURRENT_VU_DEST ], d_vf_reg[CURRENT_FT], d_dest_field[ CURRENT_VU_DEST ],
										 d_vf_reg[CURRENT_FS], d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_MTIR	(void)   {
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: VMTIR %s, %s.%s", (long)VectorUnit1.PC-8,
		debug_vu_ccr_reg[CURRENT_VU_IT],
		d_vf_reg[CURRENT_FS], 			d_fstf_field[ CURRENT_VU_FSF ]
		); 
#endif
}


void dVU1_RGET	(void)	{

#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: RGET.%  %s.%s, R ", (long)VectorUnit1.PC-8,
		d_dest_field[ CURRENT_VU_DEST ], 
		d_vf_reg[CURRENT_FT], 			d_fstf_field[ CURRENT_VU_FSF ]
		); 
#endif
}


void dVU1_RINIT	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: RINIT R  %s.%s ", (long)VectorUnit1.PC-8,
		d_vf_reg[CURRENT_FS], 			d_fstf_field[ CURRENT_VU_FSF ]
		); 
#endif
}


void dVU1_RNEXT	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: RNEXT.%  %s.%s, R ", (long)VectorUnit1.PC-8,
		d_dest_field[ CURRENT_VU_DEST ], 
		d_vf_reg[CURRENT_FT], 			d_fstf_field[ CURRENT_VU_FSF ]
		); 
#endif
}


void dVU1_RSQRT	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: SQRT Q  %s.%s %s.%s", (long)VectorUnit1.PC-8,
		d_vf_reg[CURRENT_FS], 			d_fstf_field[ CURRENT_VU_FSF ],
		d_vf_reg[CURRENT_FT], 			d_fstf_field[ CURRENT_VU_FTF ]
		); 
#endif
}


void dVU1_RXOR	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: RXOR R  %s.%s ", (long)VectorUnit1.PC-8,
		d_vf_reg[CURRENT_FS], 			d_fstf_field[ CURRENT_VU_FSF ]
		); 
#endif
}


void dVU1_SQ		(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: LQ.%s  %s.%s, 0x%4.4x(%s) ", (long)VectorUnit1.PC-8,
		d_dest_field[ CURRENT_VU_DEST ], 

		d_vf_reg[CURRENT_FS], 			d_dest_field[ CURRENT_VU_DEST ],
		(_VU_OPCODE_ & 0x7FF),
		debug_vu_ccr_reg[CURRENT_VU_IT]
		); 
#endif
}


void dVU1_SQD	(void)	{ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: SQD.%s  %s.%s, (--%s).%s ", (long)VectorUnit1.PC-8,
		d_dest_field[ CURRENT_VU_DEST ], 

		d_vf_reg[CURRENT_FS], 			d_dest_field[ CURRENT_VU_DEST ],
		debug_vu_ccr_reg[CURRENT_VU_IT],d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_SQI	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: SQI.%s  %s.%s, (%s++).%s ", (long)VectorUnit1.PC-8,
		d_dest_field[ CURRENT_VU_DEST ], 

		d_vf_reg[CURRENT_FS], 			d_dest_field[ CURRENT_VU_DEST ],
		debug_vu_ccr_reg[CURRENT_VU_IT],d_dest_field[ CURRENT_VU_DEST ]
		); 
#endif
}


void dVU1_SQRT	(void)	{
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: SQRT Q  %s.%s", (long)VectorUnit1.PC-8,
		d_vf_reg[CURRENT_FT], 			d_fstf_field[ CURRENT_VU_FTF ]
		); 
#endif
}


void dVU1_WAITQ	(void)	{ 
#ifdef VU_COP2_LOG
	VU_COP2_Log( "%8.8llx: VWAITQ  ", 
		(long)VectorUnit1.PC-8   );
#endif
}


void dVU1_WAITP	(void)	{ 
#ifdef VU_COP2_LOG
	VU_COP2_Log( "%8.8llx: VWAITP  ", 
		(long)VectorUnit1.PC-8   );
#endif
}


void dVU1_XITOP(void){
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: XITOP %s ", (long)VectorUnit1.PC-8,
		debug_vu_ccr_reg[CURRENT_VU_IT]
	); 
#endif
}


void dVU1_XTOP	(void)	{	
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: XTOP %s ", (long)VectorUnit1.PC-8,
		debug_vu_ccr_reg[CURRENT_VU_IT]
	); 
#endif
}


void dVU1_XGKICK(void){ 
#ifdef VU_MPG_LOG
	VU_MPG_LOG("%4.4llx: XGKICK %s ", (long)VectorUnit1.PC-8,
		debug_vu_ccr_reg[CURRENT_VU_IS]
	); 
#endif
}





