//---------------------------------------------------------------------------

#ifndef DbgpH
#define DbgpH


/*
 * dbgp
 */

typedef struct _DBGPHEAD{
    unsigned short id;
    unsigned char  type;
    unsigned char  code;
    unsigned char  result;
    unsigned char  count;
    unsigned short unused;
} DBGPHEAD;

#define DBGPHEAD_SZ	sizeof(DBGPHEAD)

typedef struct _DBGP_CONF{
    unsigned int   major_ver;
    unsigned int   minor_ver;
    unsigned int   target_id;
    unsigned int   reserved1;
    unsigned int   mem_align;
    unsigned int   reserved2;
    unsigned int   reg_size;
    unsigned int   nreg;
    unsigned int   nbrkpt;
    unsigned int   ncont;
    unsigned int   nstep;
    unsigned int   nnext;
    unsigned int   mem_limit_align;
    unsigned int   mem_limit_size;
    unsigned int   run_stop_state;
    unsigned int   hdbg_area_addr;
    unsigned int   hdbg_area_size;
} DBGP_CONF;

#define DBGP_CONF_SZ	sizeof(DBGP_CONF)

typedef struct _DBGP_REG{
    unsigned char  kind;
    unsigned char  number;
    unsigned short reserved;
} DBGP_REG;

#define DBGP_REG_SZ	sizeof(DBGP_REG)

typedef struct _DBGP_MEM{
    unsigned char  space;
    unsigned char  align;
    unsigned short reserved;
    unsigned int   address;
    unsigned int   length;
} DBGP_MEM;

#define DBGP_MEM_SZ	sizeof(DBGP_MEM)

typedef struct _DBGP_RUN{
    unsigned int entry;
    unsigned int gp;
    unsigned int reserved1;
    unsigned int reserved2;
    unsigned int argc;
} DBGP_RUN;

#define DBGP_RUN_SZ	sizeof(DBGP_RUN)

/* DBGP - id */
#define DBGP_CPUID_CPU	0	/* CPU (ESDBGP) */
#define DBGP_CPUID_VU0	1	/* VU0 (ESDBGP) */
#define DBGP_CPUID_VU1	2	/* VU1 (ESDBGP) */

/* DBGP - type, code */
#define DBGP_TYPE_GETCONF	0x00	/* Get Configuration Request    */
#define DBGP_TYPE_GETCONFR	0x01	/* Get Configuration Reply      */
#define DBGP_TYPE_GETREG	0x04	/* Get Register Request         */
#define DBGP_TYPE_GETREGR	0x05	/* Get Register Reply           */
#define DBGP_TYPE_PUTREG	0x06	/* Put Register Request         */
#define DBGP_TYPE_PUTREGR	0x07	/* Put Register Reply           */
#define DBGP_TYPE_RDMEM		0x08	/* Read Memory Request          */
#define DBGP_TYPE_RDMEMR	0x09	/* Read Memory Reply            */
#define DBGP_TYPE_WRMEM		0x0a	/* Write Memory Request         */
#define DBGP_TYPE_WRMEMR	0x0b	/* Write Memory Reply           */
#define DBGP_TYPE_GETBRKPT	0x10	/* Get Breakpoint Request       */
#define DBGP_TYPE_GETBRKPTR	0x11	/* Get Breakpoint Reply         */
#define DBGP_TYPE_PUTBRKPT	0x12	/* Put Breakpoint Request       */
#define DBGP_TYPE_PUTBRKPTR	0x13	/* Put Breakpoint Reply         */
#define DBGP_TYPE_BREAK		0x14	/* Break Request                */
#define DBGP_TYPE_BREAKR	0x15	/* Break Reply                  */
#define     DBGP_CODE_OTHER	  0xff	/* not-DBGP_TYPE_CONTINUE	*/
#define DBGP_TYPE_CONTINUE	0x16	/* Continue Request             */
#define   DBGP_CODE_CONT	  0x0	/* Continue                     */
#define   DBGP_CODE_STEP	  0x1	/* Step                         */
#define   DBGP_CODE_NEXT	  0x2	/* Next                         */

#define   DBGP_CODE_CONT_ERROR	  	0xF3	/**/
#define   DBGP_CODE_CONTR	  		0xF0	/* Continue                     */
#define   DBGP_CODE_STEPR	  		0xF1	/* Step                         */
#define   DBGP_CODE_NEXTR	  		0xF2	/* Next                         */


#define DBGP_TYPE_CONTINUER	0x17	/* Continue Reply		*/
#define DBGP_TYPE_RUN		0x18	/* Run Request (ESDBGP only)	*/
#define DBGP_TYPE_RUNR		0x19	/* Run Reply (ESDBGP only)	*/

/* DBGP - result */
#define DBGP_RESULT_GOOD	    0x00 /* Good                                 */
#define DBGP_RESULT_INVALIREQ	    0x01 /* Invalid Request                      */
#define DBGP_RESULT_UNIMPREQ	    0x02 /* Unimplemented Request                */
#define DBGP_RESULT_ERROR	    0x03 /* Error                                */
#define DBGP_RESULT_INVALCONT	    0x04 /* Invalid Continue                     */
#define DBGP_RESULT_TLBERR	    0x10 /* TLB mod/load/store while cmd exec    */
#define DBGP_RESULT_ADRERR	    0x11 /* Address Error (DBGT_XXMEM)           */
#define DBGP_RESULT_BUSERR	    0x12 /* Bus Error (DBGT_XXMEM)               */
#define DBGP_RESULT_INVALSTATE	    0x20 /* Invalid State (DBGT_{BREAK,CONT,..}) */
#define DBGP_RESULT_BREAKED	    0x21 /* Breaked (DBGP_TYPE_BREAK)            */
#define DBGP_RESULT_BRKPT	    0x22 /* Breakpoint (DBGT_{CONT,RUN})         */
#define DBGP_RESULT_STEPNEXT	    0x23 /* Step or Next (DBGC_{STEP,NEXT})      */
#define DBGP_RESULT_EXCEPTION	    0x24 /* Exception                            */
#define DBGP_RESULT_PROGEND	    0x25 /* normal end of program (EE)		 */
#define DBGP_RESULT_BUSYSTATE	    0x26 /* busy/critical state			 */
#define DBGP_RESULT_DEBUG_EXCEPTION 0x27 /* Debug Exception			 */

/* DBGP_CONF - major_ver */
#define DBGCF_MAJOR_CURRENT	0	/* Current Specification */

/* DBGP_CONF - minor_ver */
#define DBGCF_MINOR_FIRST	0	/* First Implementation */

/* DBGP_CONF - reg_size */
#define DBGP_CF_REG_SIZE_WORD	5	/* (1 << 5) =  32bit (IOP) */
#define DBGP_CF_REG_SIZE_QUAD	7	/* (1 << 7) = 128bit (EE)  */

/* DBGP_CONF - run_stop_state */
#define DBGP_CF_RSS_RUNNING	1	/* running */
#define DBGP_CF_RSS_STOPPED	2	/* stopped */


/* IOP register kind, number */
#define		ISDBGP_KIND_HL	1	// hi,lo
#define    	ISDBGP_KIND_GPR	2	// general purpose register
#define    	ISDBGP_KIND_SCC	3	// system control register
#define    	ISDBGP_KIND_GTER	6	// GTE register
#define    	ISDBGP_KIND_GTEC	7	// GTE control register

#define   	ISDBGP_NUM_LO	  0
#define     ISDBGP_NUM_HI	  1


/* EE register kind, number */

#define    ESDBGP_KIND_GPR	0	// general purpose register (128bit x 32)
#define    ESDBGP_KIND_HLS	1	// hi,lo,sa
#define    ESDBGP_KIND_SCR	2	// system control register (32bit x 32)
#define    ESDBGP_KIND_PCR	3	// performance counter (32bit x 3)
#define    ESDBGP_KIND_HDR	4	// hardware debug register (32bit x 8)
#define    ESDBGP_KIND_FPR	5	// floating point register (32bit x 32)
#define    ESDBGP_KIND_FPC	6	// floating point control (32bit x 2)
#define    ESDBGP_KIND_V0F	7	// VU0 floating point register (128bit x 32)
#define    ESDBGP_KIND_V0I	8	// VU0 integer/control register (32bit x 32)
#define    ESDBGP_KIND_V1F	9	// VU1 floating point register (128bit x 32)
#define    ESDBGP_KIND_V1I	10	// VU1 integer/control register (32bit x 32)

#define    ESDBGP_NUM_HI	  0	//   hi  (64bit)
#define    ESDBGP_NUM_LO	  1	//   lo  (64bit)
#define    ESDBGP_NUM_HI1	  2	//   hi1 (64bit)
#define    ESDBGP_NUM_LO1	  3	//   lo1 (64bit)
#define    ESDBGP_NUM_SA	  4	// 	 shift amount (32bit)

#define        DBGP_SPACE_MAIN_MEMORY  0	// main memory
#define        DBGP_SPACE_VU_MEM       1	// VU memory(ESDBGP)
#define        DBGP_SPACE_VU_UMEM      2	// VU micro memory(ESDBGP)

#define   DBGP_ALIGN_BYTE		0
#define   DBGP_ALIGN_HALFWORD	1
#define   DBGP_ALIGN_WORD		2
#define   DBGP_ALIGN_DOUBLE		3
#define   DBGP_ALIGN_QWORD		4
#define   DBGP_ALIGN_1KB		10





#define		DBGP_LOG	DBGP_Log
//#undef      DBGP_LOG


void	DecodeESDbgp( );
void	CreatePacket_ESDbgp ( int fCode , int fParam) ;

void	DecodeISDbgp( ) ;
void	CreatePacket_ISDbgp ( int fCode , int fParam) ;

void 	SendPacket_Dbgp( int fProto);


void 	SendPacket_Dbgp_Breaked( );
void 	SendPacket_Dbgp_EndProgram( );

void DBGP_Log ( char * ftext  , ...   ) ;

//---------------------------------------------------------------------------
#endif
