/**
 * nSX2 - an OpenSource PS2 Emulator
 * Copyright (C) 2002 Muad
 *
 * homepage: http://nsx2.emulation64.com
 * email address: atreides.muaddib@caramail.com
 * 
 * If you want to contribute to the project please contact
 * me first (maybe someone is already making what you are
 * planning to do).
 *
 *
 * This program is free software; you can redistribute it and/
 * or modify it under the terms of the GNU General Public Li-
 * cence as published by the Free Software Foundation; either
 * version 2 of the Licence, or any later version.
 *
 * This program is distributed in the hope that it will be use-
 * ful, but WITHOUT ANY WARRANTY; without even the implied war-
 * ranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public Licence for more details.
 *
 * You should have received a copy of the GNU General Public
 * Licence along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
**/
#include <stdio.h>
#include <stdlib.h>

#include <strings.h>
#include <float.h>
#include <math.h>


#include "nSX2.h"

#include "r5900_main.h"
 
#include "bios_syscalls.h"
#include "bios_syscalls_table.h"
 
#include "PS2Edefs.h"

#include "Sif.h"

/* I must declare a function to avoid this kind of include */
#include "DmaC.h"

/* must create function to avoid this kind of declaration */
#include "IntrC.h"


#include "GS.h"

/**********************************************************************************************************
							INTERRUPTS BY CAUSE		
***********************************************************************************************************/

char * debugIntrptCause[ 11 ] = {"GS", "SBUS", "V-blank start", "V-blank end", "VIF0", "VIF1", "VU0", "VU1", "IPU", "Timer0", "Timer1" };


/*
_EnableIntc	ENTRY_20	800008c0

Description:This system call enables interrupts triggered by cause.

Param: 
AO : Cause

Return value
0 Already enabled
1 Changed to enabled

*/
void SYSCALL__EnableIntc( ) { 
	char 	temp_string[256];	//Debug Text
    _u8		cause;				//Cause ID
          
    _u32 newValue;				//New Value of Interrupt by Cause

	cause = MipsCpu.GPR[CPU_REG_A0].lo;

	// Show Debug Info 
#ifdef SYSCALL_LOG
	SYSCALL_LOG("%8.8lx:[BIOS]_EnableIntc : cause n %ld ( %s )",(long)MipsCpu.PC,cause, debugIntrptCause[cause] );
#endif

	
	
	/* Read the Interrupt Mask register */
   newValue = ( 0x1 << MipsCpu.GPR[CPU_REG_A0].lo ) & IntC.Reg->mask ;
 
    
    //If the Interrupt is already Enable return 0
    if ( newValue != 0 ) {
         MipsCpu.GPR[CPU_REG_V0].lo =  0;
     }
    //Else enable the Interrupt and return 1
    else {
        IntC.Reg->mask = IntC.Reg->mask | ( 0x1 << MipsCpu.GPR[CPU_REG_A0].lo );
       
		MipsCpu.GPR[CPU_REG_V0].lo =  1;
    }
		
	INCREMENT_PC();  
	
}   

/*
_DisableIntc	ENTRY_21	80000900
*/
void SYSCALL__DisableIntc( ) 					{ 
	char 	temp_string[256];	//Debug Text
    _u8		cause;				//Cause ID
          
    _u32 newValue;				//New Value of Interrupt by Cause

	cause = MipsCpu.GPR[CPU_REG_A0].lo;

	/* Show Debug Info */
#ifdef SYSCALL_LOG
	SYSCALL_LOG("%8.8lx:[BIOS]_DisableIntc : cause n %ld ( %s )",(long)MipsCpu.PC, cause, debugIntrptCause[cause] );
#endif

	
	/* Read the Interrupt Mask register */
   newValue = ( 0x1 << MipsCpu.GPR[CPU_REG_A0].lo ) & IntC.Reg->mask ;

		
     /* Write new Value to INTC_MASK register */ 
	IntC.Reg->mask = IntC.Reg->mask & ~( 0x1 << MipsCpu.GPR[CPU_REG_A0].lo );

       
	MipsCpu.GPR[CPU_REG_V0].lo =  1;
    
		
	INCREMENT_PC();  
	
   

}   


void SYSCALL__iDisableIntc( ) 					{ 
#ifdef SYSCALL_LOG
	SYSCALL_LOG("%8.8lx:[BIOS]_iDisableIntc : NI",(long)MipsCpu.PC );
#endif
	MipsCpu.halt = 1; 
}     


void SYSCALL__iEnableIntc( ) 					{ 
#ifdef SYSCALL_LOG
	SYSCALL_LOG("%8.8lx:[BIOS]_iEnableIntc : NI",(long)MipsCpu.PC );
#endif
	MipsCpu.halt = 1; 
}     


/**********************************************************************************************************
				INTERRUPT BY CAUSE HANDLER								
***********************************************************************************************************/

/*
AddIntcHandler	ENTRY_16	80001a30


Description: Registers an INTC handler by interrupt cause.
cause is the interrupt cause. 

handler is the address of the handler that will be called when an interrupt
is generated, and will be passed the interrupt cause in the argument ca. 

next is the handler ID of a previously registered handler. 
If next is 0, the handler will be registered as the first handler.
If next is -1, the handler will be registered as the last handler.

Note: When a INTC interrupt is generated, the GPRs are saved before the interrupt handler is called. When
the handler has finished executing, the GPRs are restored. However, note that there is no save/restore of
FPU registers.
When writing interrupt handlers, note that interrupts are disabled and that different APIs can be used. 
Referto \overview\eekernel for information on these issues.

If an interrupt handler returns -1, subsequently registered interrupt handlers will not be called.

REGISTER RETURN VALUE : V0


Param:
A0 : Cause ID
A1 : Handler Adress
A2 : Next ( ?, not sure of this )

Return:
Handler ID  :Normal termination
-1 			:Error return

*/
void SYSCALL_AddIntcHandler( ) 					{ 
	char temp_string[256];
	
	_u8		CauseID;
	_u32	IntcHandlerAddr;
	_u32	IntcNextRegisterHandler;

	CauseID  				= MipsCpu.GPR[CPU_REG_A0].lo & 0xFF;
	IntcHandlerAddr 		= MipsCpu.GPR[CPU_REG_A1].lo;
	IntcNextRegisterHandler = MipsCpu.GPR[CPU_REG_A2].lo;

#ifdef SYSCALL_LOG
	SYSCALL_LOG("%8.8lx:[BIOS]AddIntcHandler : cause n %ld ( %s ), Add : 0x%8.8lx, next : 0x%8.8lx ",
					(long)MipsCpu.PC,CauseID, debugIntrptCause[CauseID], IntcHandlerAddr, IntcNextRegisterHandler );
#endif

	MipsCpu.GPR[CPU_REG_V0].lo =  BIOS_AddINTCHandler ( CauseID, IntcHandlerAddr,IntcNextRegisterHandler ) ;

	INCREMENT_PC(); 
}  

/*
RemoveIntcHandler	ENTRY_17	80001a70
*/
void SYSCALL_RemoveIntcHandler( ) 				{ 

	char temp_string[256];
	
	_u8		CauseID;
	_u32	IntcHandlerAddr;
	_u32	IntcNextRegisterHandler;

	CauseID  				= MipsCpu.GPR[CPU_REG_A0].lo & 0xFF;
	IntcHandlerAddr 		= MipsCpu.GPR[CPU_REG_A1].lo;
	IntcNextRegisterHandler = MipsCpu.GPR[CPU_REG_A2].lo;

#ifdef SYSCALL_LOG
	SYSCALL_LOG("%8.8lx:[BIOS]RemoveIntcHandler : cause n %ld ( %s ), Add : 0x%8.8lx, next : 0x%8.8lx ",
					(long)MipsCpu.PC,CauseID, debugIntrptCause[CauseID], IntcHandlerAddr, IntcNextRegisterHandler );
#endif


	MipsCpu.GPR[CPU_REG_V0].lo = BIOS_RemoveIntcHandler(CauseID, IntcHandlerAddr );

	INCREMENT_PC(); 


}  


void SYSCALL_EnableIntcHandler		( void )	{ 
#ifdef SYSCALL_LOG
	SYSCALL_LOG("%8.8lx:[BIOS]EnableIntcHandler : NI",(long)MipsCpu.PC );
#endif	
	MipsCpu.halt = 1; 
}   



void 	SYSCALL_DisableIntcHandler	( void )	{ 
#ifdef SYSCALL_LOG
	SYSCALL_LOG("%8.8lx:[BIOS]DisableIntcHandler : NI",(long)MipsCpu.PC );
#endif	
	MipsCpu.halt = 1; 
}   



void 	SYSCALL_iDisableIntcHandler	( void )	{ 
#ifdef SYSCALL_LOG
	SYSCALL_LOG("%8.8lx:[BIOS]iDisableIntcHandler : NI",(long)MipsCpu.PC );
#endif	
	MipsCpu.halt = 1; 
}     


void 	SYSCALL_iEnableIntcHandler	( void )	 { 
#ifdef SYSCALL_LOG
	SYSCALL_LOG("%8.8lx:[BIOS]iEnableIntcHandler : NI",(long)MipsCpu.PC );
#endif
	MipsCpu.halt = 1; 
}     


/******************************************************************************/

_u32 BIOS_AddINTCHandler ( _u32 fCause, _u32 fHandlerAdd, _u32 fNext ) {

	_u32	ReturnValue;

	LinkedAddress * FirstHandlerAdd;
	LinkedAddress * CurrentHandlerAdd;
	LinkedAddress * SavedHandlerAdd;

	FirstHandlerAdd 	= IntC.IntByCause[ fCause ].HandlerAdd;

	/* How to Register... */
	switch ( (_u32)fNext ) {
		/* Registered as first handler */
		case 0 :
			/* Save the First Handler ptr */
			SavedHandlerAdd = FirstHandlerAdd;				

			/* Alloc Memory for Handler */
			CurrentHandlerAdd = ( LinkedAddress * ) malloc ( sizeof ( LinkedAddress ) );

			/* The Handler is Enable */
			IntC.IntByCause[ fCause ].Status 	= 1 ;
		
			/* The handler has an ID */
			CurrentHandlerAdd->ID	= IntC.IntByCause[ fCause ].Numero_ID;
			CurrentHandlerAdd->Add	= (_u32) fHandlerAdd;

			/* Next become the former First Handler */
			CurrentHandlerAdd->next = SavedHandlerAdd;

			/* Set New Handler as First */
			IntC.IntByCause[ fCause ].HandlerAdd 	= CurrentHandlerAdd;

			/* Update Common Handler Var */
			IntC.IntByCause[ fCause ].Numero_ID		= IntC.IntByCause[ fCause ].Numero_ID + 1;
			IntC.IntByCause[ fCause ].Nb_handler	= IntC.IntByCause[ fCause ].Nb_handler + 1;

			/* return ID */
			ReturnValue = CurrentHandlerAdd->ID;

		break;

		/* Registred as Last handler */
		case (_u32)0xFFFFFFFF :
			SavedHandlerAdd = FirstHandlerAdd;

			/* The handler is enable */
			IntC.IntByCause[ fCause ].Status 	= 1 ;

			if ( SavedHandlerAdd == NULL ) {

				IntC.IntByCause[ fCause ].HandlerAdd = ( LinkedAddress * ) malloc ( sizeof ( LinkedAddress ) );
				CurrentHandlerAdd = IntC.IntByCause[ fCause ].HandlerAdd;

			}else {

				/* Find the last Handler */
				while ( SavedHandlerAdd->next != NULL )
					SavedHandlerAdd = SavedHandlerAdd->next;

				/* Alloc Memory for the Handler */
				SavedHandlerAdd->next = ( LinkedAddress * ) malloc ( sizeof ( LinkedAddress ) );
			
				/* Put Current at the end of List */
				CurrentHandlerAdd = SavedHandlerAdd->next;

			}

			CurrentHandlerAdd->Add	= (_u32) fHandlerAdd;
			CurrentHandlerAdd->ID	= IntC.IntByCause[ fCause ].Numero_ID;
			
			/* No more handler after */
			CurrentHandlerAdd->next = NULL;
			
			/* Update Common Handler Var */	
			IntC.IntByCause[ fCause ].Numero_ID		= IntC.IntByCause[ fCause ].Numero_ID + 1;
			IntC.IntByCause[ fCause ].Nb_handler	= IntC.IntByCause[ fCause ].Nb_handler + 1;

			/* return ID */
			ReturnValue = CurrentHandlerAdd->ID;
	
		break;

		/* Register after next */
		default:

#ifdef SYSCALL_LOG
			SYSCALL_LOG("%8.8lx:[BIOS]AddINTCHandler : UnHandle Case Next",(long)MipsCpu.PC );
#endif	
			MipsCpu.halt = 1;
	
			ReturnValue = (_u32)0xFFFFFFFF;
		break;

	}

	return ReturnValue;
}


_u32	BIOS_RemoveIntcHandler(_u32 fCause, _u32 fHandlerID ) {

	_u8		FoundHandler = FALSE;
	_u32	ReturnValue;


	LinkedAddress * PreviousHandlerAdd;	
	LinkedAddress * CurrentHandlerAdd;
	LinkedAddress * NextHandlerAdd;	

	PreviousHandlerAdd 	= IntC.IntByCause[ fCause ].HandlerAdd;
	CurrentHandlerAdd 	= IntC.IntByCause[ fCause ].HandlerAdd;
	if ( CurrentHandlerAdd != NULL ) NextHandlerAdd		= CurrentHandlerAdd->next;


	while ( CurrentHandlerAdd != NULL ) {


		if ( CurrentHandlerAdd->ID == fHandlerID ) {

			if ( PreviousHandlerAdd != NULL ) PreviousHandlerAdd->next = NextHandlerAdd;
			
			free ( CurrentHandlerAdd );
			CurrentHandlerAdd = NULL;

			IntC.IntByCause[ fCause ].Nb_handler = IntC.IntByCause[ fCause ].Nb_handler - 1;

			if ( IntC.IntByCause[ fCause ].Nb_handler == 0 ) IntC.IntByCause[ fCause ].HandlerAdd = NULL;

			FoundHandler = TRUE;
		}
		else {

			PreviousHandlerAdd = CurrentHandlerAdd;
			if ( CurrentHandlerAdd != NULL ) CurrentHandlerAdd = CurrentHandlerAdd->next;
			if ( CurrentHandlerAdd != NULL ) NextHandlerAdd = CurrentHandlerAdd->next;

		}


	}


	if ( FoundHandler == TRUE )  
		ReturnValue	= IntC.IntByCause[ fCause ].Nb_handler;
	else 
		ReturnValue	= (_u32)0xFFFFFFFF;
	

	return ReturnValue;
	
}


